#ifndef RICHMIRRCOMBINFIT_MIRRCOMBFIT_H
#define RICHMIRRCOMBINFIT_MIRRCOMBFIT_H

//Local
#include "RichMirrCombinFit/IMirrCombFit.h"
#include "RichMirrCombinFit/MirrComb.h"
//C++
#include <iostream>
#include <string>
#include <map>
#include <vector>
//ROOT
#include <TFile.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TString.h>

using namespace std;

typedef map<string, MirrComb> MapCombNamCombObj;

class MirrCombFit : public IMirrCombFit
{
public:

  MirrCombFit();
  virtual ~MirrCombFit();

  void         MirrCombList();

  void         HistoList();

  string       MirrCombNam(int pri, int sec);


  void ReadConfFile(const string& confFile) override;

  void FitHistos() override;

  void StopOrContinue() override;

  void CombinFitDecision() override;

  void PrintResults() override;

  void GlobalFit() override;


  virtual void GetFitParams(TH2D*) = 0;


  double       GetGlobalFitMean(TH1D* histo);

  void         SetShift(double shift) override;

  void         BinCounter(TH2D*);

  void         Print(ostream& out) const;


private:

  void         OpenFile();

  TFile* f = nullptr;


protected:

  void         CreateMapCombNamCombObj();


  // ATTENTION: for observability, please any future
  // additions keep in the alphabetic order !!!
  int                      m_backgroundOrder{0};
  double                   m_begRange{0};
  string                   m_combAndMirrSubsetsFile;
  double                   m_deltaThetaWindow{0};
  double                   m_endRange{0};
  int                      m_fitMethod{0};
  int                      m_fixSinusoidShift{0};
  vector<TString>          m_histoNames;
  string                   m_inputHistosFile;
  int                      m_iterationCount{0};
  MapCombNamCombObj        m_mapNamObj;
  int                      m_minAverageBinPop{0};
  vector<string>           m_mirrCombList;
  double                   m_nominalResolutionSigma{0};
  double                   m_nominSig{0};
  string                   m_outputResultsFile;
  int                      m_phiBinFactor{0};
  string                   m_plotDir;
  int                      m_plotOutputLevel{0};
  int                      m_priMirrs{0};
  string                   m_rich{0};
  int                      m_secMirrs{0};
  double                   m_sinusoidShift{0};
  double                   m_stopSigmaFraction{0};
  double                   m_stopTolerance{0};
  int                      m_stopToleranceMode{0};
  string                   m_thisVariant;
  int                      m_useGlobalFitMean{0};
  double                   m_warningFactor{0};
  int                      m_zeroGlobalFitMean{0};

};

ostream& operator<<(ostream& out, const MirrCombFit& fit);

#endif
