from ROOT import TH1D, TCanvas, TFile, TF1, gStyle, TGraph, TMultiGraph, TLine, TH2D
import numpy

class AlignSummary:
    def __init__(self, _alignConf, _maxIt, ):
        self.alignConf = _alignConf
        self.nameStr = self.alignConf.getProp('nameStr')
        self.whichRich = self.alignConf.getProp('Rich')
        self.workdir = self.alignConf.getProp('WorkDir')
        self.maxIt = _maxIt # the maximum labeled number of the iterations that we are looping through
        self.prename =  "RichRecQCHistos_rich" + str(self.whichRich) + "_" + self.nameStr + "_"
        self.fitMin = -0.007 # default: RICH1
        self.fitMax = 0.007 # default: RICH1
        if (self.whichRich == 2):
            self.fitMin = -0.0039
            self.fitMax = 0.0035
        self.maxpri = 3 # default: RICH1
        self.binToPriMirr = [[2,3],
                             [0,1]]
        self.maxsec = 15 # default: RICH1
        self.binToSecMirr = [[10,11,14,15],
                             [8,9,12,13],
                             [2,3,6,7],
                             [0,1,4,5]]
        if (self.whichRich == 2):
            self.maxpri = 55
            self.binToPriMirr = [[3,2,1,0,31,30,29,28],
                                 [7,6,5,4,35,34,33,32],
                                 [11,10,9,8,39,38,37,36],
                                 [15,14,13,12,43,42,41,40],
                                 [19,18,17,16,47,46,45,44],
                                 [23,22,21,20,51,50,49,48],
                                 [27,26,25,24,55,54,53,52]]
            self.maxsec = 39
            self.binToSecMirr = [[3,2,1,0,23,22,21,20],
                                 [7,6,5,4,27,26,25,24],
                                 [11,10,9,8,31,30,29,28],
                                 [15,14,13,12,35,34,33,32],
                                 [19,18,17,16,39,38,37,36]]
    
    def makeSummaryPlot(self, compareXML):
        import os
        gStyle.SetOptFit(1111)
        gStyle.SetOptStat(000000000)
                
        saveName = self.workdir + 'Rich' + str(self.whichRich) + '_AlignSummary.pdf'
        cPlot = TCanvas('cPlot', 'ckRes')
        cPlot.SaveAs(saveName + '[')

        # heatHistos
        cPlot.Divide(2,2,0.001,0.001)
        from tiltObj import tiltObj
        mirrtilts = []
        for its in range(0, self.maxIt + 1):
            XMLFile = self.workdir + 'Rich' + str(self.whichRich) + 'CondDBUpdate_' + self.nameStr + '_i' + str(its + 1) + '.xml' 
            mirrtilts.append( tiltObj(self.alignConf) )
            mirrtilts[its].setChange([compareXML, XMLFile])
        mgraphs = mirrtilts[self.maxIt].getGraphs()
        
        if (self.whichRich == 1):
            heatHistoPriY = TH2D('heatPriY', 'RICH' + str(self.whichRich) + ' Primary mirrors local Y rotation (mrad)', 2, -1, 1, 2, -1, 1)
            heatHistoPriZ = TH2D('heatPriZ', 'RICH' + str(self.whichRich) + ' Primary mirrors local Z rotation (mrad)', 2, -1, 1, 2, -1, 1)
            heatHistoSecY = TH2D('heatSecY', 'RICH' + str(self.whichRich) + ' Secondary mirrors local Y rotation (mrad)', 4, -1, 1, 4, -1, 1)
            heatHistoSecZ = TH2D('heatSecZ', 'RICH' + str(self.whichRich) + ' Secondary mirrors local Z rotation (mrad)', 4, -1, 1, 4, -1, 1)
        if (self.whichRich == 2):
            heatHistoPriY = TH2D('heatPriY', 'RICH' + str(self.whichRich) + ' Primary mirrors local Y rotation (mrad)', 8, -1, 1, 7, -1, 1)
            heatHistoPriZ = TH2D('heatPriZ', 'RICH' + str(self.whichRich) + ' Primary mirrors local Z rotation (mrad)', 8, -1, 1, 7, -1, 1)
            heatHistoSecY = TH2D('heatSecY', 'RICH' + str(self.whichRich) + ' Secondary mirrors local Y rotation (mrad)', 8, -1, 1, 5, -1, 1)
            heatHistoSecZ = TH2D('heatSecZ', 'RICH' + str(self.whichRich) + ' Secondary mirrors local Z rotation (mrad)', 8, -1, 1, 5, -1, 1)        
        
        xTitle = 'relative global X position (not to scale)'
        yTitle = 'relative global Y position (not to scale)'
        heatHistoPriY.SetXTitle(xTitle)
        heatHistoPriY.SetYTitle(yTitle)     
        heatHistoSecY.SetXTitle(xTitle)
        heatHistoSecY.SetYTitle(yTitle)   
        heatHistoPriZ.SetXTitle(xTitle)
        heatHistoPriZ.SetYTitle(yTitle)     
        heatHistoSecZ.SetXTitle(xTitle) 
        heatHistoSecZ.SetYTitle(yTitle) 

        heatHistoPriY.SetMarkerSize(2.0*heatHistoPriY.GetMarkerSize())
        heatHistoPriZ.SetMarkerSize(2.0*heatHistoPriZ.GetMarkerSize())
        heatHistoSecY.SetMarkerSize(2.0*heatHistoSecY.GetMarkerSize())
        heatHistoSecZ.SetMarkerSize(2.0*heatHistoSecZ.GetMarkerSize())

        gStyle.SetPaintTextFormat('5.3f') # formerly 4.2f

        self.colorsLow  = [923,921,0,390,622,624,627,631,636,635,634,633,632,807,797,800,400,416]
        self.colorsHigh = [416,432,867,857,600,616,617,618,619,620,615,611,608,606,590,920,922,1]
        colorsList = self.colorsLow + self.colorsHigh
               
        self.sTPY = float(self.alignConf.getProp('stopTolerancePriY'))
        self.sTPZ = float(self.alignConf.getProp('stopTolerancePriZ'))
        self.sTSY = float(self.alignConf.getProp('stopToleranceSecY'))
        self.sTSZ = float(self.alignConf.getProp('stopToleranceSecZ'))

        for its in reversed(range(0, self.maxIt + 1)):
            heatHistoPriY.SetTitle('RICH' + str(self.whichRich) + ' Primary mirrors local Y rotation (mrad) after It. ' + str(its) )
            heatHistoPriZ.SetTitle('RICH' + str(self.whichRich) + ' Primary mirrors local Z rotation (mrad) after It. ' + str(its) )
            heatHistoSecY.SetTitle('RICH' + str(self.whichRich) + ' Secondary mirrors local Y rotation (mrad) after It. ' + str(its) )
            heatHistoSecZ.SetTitle('RICH' + str(self.whichRich) + ' Secondary mirrors local Z rotation (mrad) after It. ' + str(its) )
            
            rowsPri = len(self.binToPriMirr)
            colsPri = len(self.binToPriMirr[0])
            for row in xrange(rowsPri):
                for col in xrange(colsPri):
                    heatHistoPriY.SetBinContent(col+1, row+1, mirrtilts[its].priY[self.binToPriMirr[row][col]]) 
                    heatHistoPriZ.SetBinContent(col+1, row+1, mirrtilts[its].priZ[self.binToPriMirr[row][col]])
    
            rowsSec = len(self.binToSecMirr)
            colsSec = len(self.binToSecMirr[0])
            for row in xrange(rowsSec):
                for col in xrange(colsSec):
                    heatHistoSecY.SetBinContent(col+1, row+1, mirrtilts[its].secY[self.binToSecMirr[row][col]] )
                    heatHistoSecZ.SetBinContent(col+1, row+1, mirrtilts[its].secZ[self.binToSecMirr[row][col]] )
    
            biggestTilt = mirrtilts[its].getAbsMax() 
                     
            if int(self.alignConf.getProp('stopToleranceMode')) == 1:
                for n in reversed(range(0,17)):
                    if biggestTilt[0] > n*self.sTPY or biggestTilt[1] > n*self.sTPZ or biggestTilt[2] > n*self.sTSY or biggestTilt[3] > n*self.sTSZ :
                        break
                heatHistoPriY.GetZaxis().SetRangeUser((n+2)*(0-self.sTPY), (n+2)*(self.sTPY))
                heatHistoPriZ.GetZaxis().SetRangeUser((n+2)*(0-self.sTPZ), (n+2)*(self.sTPZ))
                heatHistoSecY.GetZaxis().SetRangeUser((n+2)*(0-self.sTSY), (n+2)*(self.sTSY))
                heatHistoSecZ.GetZaxis().SetRangeUser((n+2)*(0-self.sTSZ), (n+2)*(self.sTSZ))
                colorsLowClone = list(self.colorsLow)
                colorsLowClone.reverse()
                colorsLowSubset = list(colorsLowClone[0:n+2])
                colorsLowSubset.reverse()
                colorsList = list(colorsLowSubset + self.colorsHigh[0:n+2])
                levelsPY = []
                levelsPZ = []
                levelsSY = []
                levelsSZ = []             
                for i in range(-(n+2),(n+2)+1):
                    levelsPY += [i*self.sTPY]
                    levelsPZ += [i*self.sTPZ]
                    levelsSY += [i*self.sTSY]
                    levelsSZ += [i*self.sTSZ]
                heatHistoPriY.SetContour(len(list(levelsPY)), numpy.array(list(levelsPY), dtype = 'float64'))
                heatHistoPriZ.SetContour(len(list(levelsPZ)), numpy.array(list(levelsPY), dtype = 'float64'))
                heatHistoSecY.SetContour(len(list(levelsSY)), numpy.array(list(levelsPY), dtype = 'float64'))
                heatHistoSecZ.SetContour(len(list(levelsSZ)), numpy.array(list(levelsPY), dtype = 'float64'))
            else:
                 # for now this is OK, however it is wrong to rely on the values of the stop tolerances in the configuration file, since those are not actually being used
                heatHistoPriY.GetZaxis().SetRangeUser(-18*self.sTPY, 18*self.sTPY) # Eventually we will have to extract these tolerances from RichMirrAlign
                heatHistoPriZ.GetZaxis().SetRangeUser(-18*self.sTPZ, 18*self.sTPZ) # Eventually we will have to extract these tolerances from RichMirrAlign
                heatHistoSecY.GetZaxis().SetRangeUser(-18*self.sTSY, 18*self.sTSY) # Eventually we will have to extract these tolerances from RichMirrAlign
                heatHistoSecZ.GetZaxis().SetRangeUser(-18*self.sTSZ, 18*self.sTSZ) # Eventually we will have to extract these tolerances from RichMirrAlign
                levelsPY = []
                levelsPZ = []
                levelsSY = []
                levelsSZ = []             
                for i in range(-(18),(18)+1):
                    levelsPY += [i*self.sTPY]
                    levelsPZ += [i*self.sTPZ]
                    levelsSY += [i*self.sTSY]
                    levelsSZ += [i*self.sTSZ]

            gStyle.SetPalette(len(list(colorsList)), numpy.array(list(colorsList), dtype = 'intc'))
            
            heatHistoPriY.SetContour(len(list(levelsPY)), numpy.array(list(levelsPY), dtype = 'float64'))
            heatHistoPriZ.SetContour(len(list(levelsPZ)), numpy.array(list(levelsPZ), dtype = 'float64'))
            heatHistoSecY.SetContour(len(list(levelsSY)), numpy.array(list(levelsSY), dtype = 'float64'))
            heatHistoSecZ.SetContour(len(list(levelsSZ)), numpy.array(list(levelsSZ), dtype = 'float64'))

            cPlot.cd(1)
            heatHistoPriY.Draw("COLZ")
            heatHistoPriY.Draw("TEXT SAME")
            cPlot.cd(2)
            heatHistoPriZ.Draw("COLZ")
            heatHistoPriZ.Draw("TEXT SAME")
            cPlot.cd(3)
            heatHistoSecY.Draw("COLZ")
            heatHistoSecY.Draw("TEXT SAME")
            cPlot.cd(4)
            heatHistoSecZ.Draw("COLZ")
            heatHistoSecZ.Draw("TEXT SAME")
            cPlot.SaveAs(saveName)
            
        # resHistograms           
        cPlot.Clear()
        if self.maxIt == 0:
            pass
        elif self.maxIt < 2:
            cPlot.Divide(2,1,0.0025,0.0025)
        elif self.maxIt < 3:
            cPlot.Divide(3,1,0.0025,0.0025)       
        elif self.maxIt < 4:
            cPlot.Divide(2,2,0.0025,0.0025)    
        elif self.maxIt < 6:
            cPlot.Divide(3,2,0.0025,0.0025)     
        elif self.maxIt < 8:
            cPlot.Divide(4,2,0.0025,0.0025)  
        elif self.maxIt < 9:
            cPlot.Divide(3,3,0.0025,0.0025)
        else:
            cPlot.Divide(4,4,0.0025,0.0025)
            
        # reserve spaces for resHistograms      
        resHistograms = [None] * (self.maxIt + 1) 
        polbkg = [None] * (self.maxIt + 1)
        help = [None] * (self.maxIt + 1)
        hist = [None] * (self.maxIt + 1)
        filename = [None] * (self.maxIt + 1)
        thisFile = [None] * (self.maxIt + 1)
        
        # define resHisto
        resHisto = TH1D('hres', 'RICH' + str(self.whichRich) + ' Cherenkov angle resolution (mrad) per It. ', self.maxIt + 1, -0.5, self.maxIt + 0.5)

        for j in reversed(range(0, self.maxIt + 1)):
            filename[j] = self.workdir + '/' + self.prename + 'i' +  str(j) + '.root'
            if os.path.exists(filename[j]):
                cPlot.cd(j+1)
                
                thisFile[j] = TFile(filename[j], 'read')
                hist[j] = thisFile[j].Get('RICH/RiCKResLongTightMirror/Rich' + str(self.whichRich) + 'Gas/ckResAll')
                title = 'RICH ' + str(self.whichRich) + ' Cherenkov angle resolution It. ' + str(j) 
                fitRes = []
                fitRes = self.fitCherenkovAngle(hist[j])
                resHistograms[j] = fitRes[0]        
                resHisto.SetBinContent(j+1, fitRes[1] * 1000)
                resHisto.SetBinError(j+1, fitRes[2] * 1000)        

                resHistograms[j].SetTitle(title)
                resHistograms[j].SetXTitle('#Delta#theta_{Cherenkov} (rad)')
                resHistograms[j].SetYTitle('Entries')
                help[j] = resHistograms[j].GetFunction("Rich" + str(self.whichRich) + "fFitF3")
                polbkg[j] = TF1("polBKG"+str(j),"pol3(0)", self.fitMin, self.fitMax)
                polbkg[j].SetParameter(0, help[j].GetParameter(3))
                polbkg[j].SetParameter(1, help[j].GetParameter(4))
                polbkg[j].SetParameter(2, help[j].GetParameter(5))
                polbkg[j].SetParameter(3, help[j].GetParameter(6))
                polbkg[j].SetLineColor(4)

                resHistograms[j].Draw()
                polbkg[j].Draw('same')
        cPlot.SaveAs(saveName)
        
        # Make resHisto iteration trend plot
        cPlot.Clear()
        resHisto.SetMarkerStyle(8)
        resHisto.SetXTitle('iteration number')
        resHisto.SetYTitle('') # resHisto.SetYTitle('Cherenkov angle resolution (mrad)')
        resHisto.Draw()
        cPlot.SaveAs(saveName)

        #maxPriY = TLine(0, self.alignConf.getProp('stopTolerancePriY'), self.maxpri, self.alignConf.getProp('stopTolerancePriY'))
        #minPriY = TLine(0, -self.alignConf.getProp('stopTolerancePriY'), self.maxpri, -self.alignConf.getProp('stopTolerancePriY')) 
        #maxPriZ = TLine(0, self.alignConf.getProp('stopTolerancePriZ'), self.maxpri, self.alignConf.getProp('stopTolerancePriZ'))                                                                                
        #minPriZ = TLine(0, -self.alignConf.getProp('stopTolerancePriZ'), self.maxpri, -self.alignConf.getProp('stopTolerancePriZ'))    
        #maxSecY = TLine(0, self.alignConf.getProp('stopToleranceSecY'), self.maxpri, self.alignConf.getProp('stopToleranceSecY'))  
        #minSecY = TLine(0, -self.alignConf.getProp('stopToleranceSecY'), self.maxpri, -self.alignConf.getProp('stopToleranceSecY'))    
        #maxSecZ = TLine(0, self.alignConf.getProp('stopToleranceSecZ'), self.maxpri, self.alignConf.getProp('stopToleranceSecZ'))                                                                                
        #minSecZ = TLine(0, -self.alignConf.getProp('stopToleranceSecZ'), self.maxpri, -self.alignConf.getProp('stopToleranceSecY'))    
        #maxPriY.SetLineColor(2) #red
        #minPriY.SetLineColor(2)
        #maxPriZ.SetLineColor(4) #blue
        #minPriZ.SetLineColor(4)
        #maxSecY.SetLineColor(2)
        #minSecY.SetLineColor(2)
        #maxSecZ.SetLineColor(4)
        #minSecZ.SetLineColor(4)
        
        # compensations graphs
        cPlot.Clear()
        mgraphs[0].Draw('AP')
        #maxPriY.Draw('same')
        #minPriY.Draw('same')
        #maxPriZ.Draw('same')
        #minPriZ.Draw('same')
        mgraphs[0].SetTitle("RICH" + str(self.whichRich) +" primary mirror compensations w.r.t. to comparison DB")
        mgraphs[0].GetXaxis().SetTitle("primary mirror number")
        mgraphs[0].GetYaxis().SetTitle("change in mirror compensation (mrad) [{red, blue}: local {Y, Z}]")
        cPlot.SaveAs(saveName)
        
        cPlot.Clear()
        mgraphs[1].Draw('AP')
        #maxSecY.Draw('same')
        #minSecY.Draw('same')
        #maxSecZ.Draw('same')
        #minSecZ.Draw('same')
        mgraphs[1].SetTitle("RICH" + str(self.whichRich) +" secondary mirror compensations w.r.t. to comparison DB")
        mgraphs[1].GetXaxis().SetTitle("secondary mirror number")
        mgraphs[1].GetYaxis().SetTitle("change in mirror compensation (mrad) [{red, blue}: local {Y, Z}]")
        cPlot.SaveAs(saveName)
        
        cPlot.SaveAs(saveName + ']')
        
        for its in range(0, self.maxIt + 1):
            mirrtilts[self.maxIt].writeChange(self.maxIt +  1)
        mgraphs[0].Delete()
        mgraphs[1].Delete()

    def fitCherenkovAngle(self, hist):
        richStr = 'Rich' + str(self.whichRich)
        m_maxErrorForOK = 1e-3;
        
        xPeak = hist.GetBinCenter( hist.GetMaximumBin() )
        delta = 0
        if (richStr == 'Rich1'):
            delta = 0.0025
        else:
            delta = 0.00105
        
        fitMin = xPeak - delta
        fitMax = xPeak + delta
        
        preFitFName = richStr + "PreFitF"
        preFitF = TF1( preFitFName, "gaus", fitMin, fitMax )
        preFitF.SetParameter(1, 0 )
        
        if (richStr == 'Rich1') :
            preFitF.SetParameter(2, 0.0015 )
        else:
            preFitF.SetParameter(2, 0.0007 )
        
        hist.Fit(preFitF, "RQ" );
        fitOK = False;
        
        lastFitF = preFitF
        bestFitF = None
        nPolFull = 3
        bestNPol = 0
        
        fFitF = None

        for nPol in range(1, nPolFull + 1):
            fFitFName = richStr + "fFitF" + str(nPol)
            fFitFType = "gaus(0)+pol"+ str(nPol)+"(3)" 
            fFitF = TF1( fFitFName, fFitFType, self.fitMin, self.fitMax )
            
            fFitF.SetParName(0,"Gaus Constant")
            fFitF.SetParName(1,"Gaus Mean")
            fFitF.SetParName(2,"Gaus Sigma")
            nParamsToSet = 3 
            if ( nPol > 1 ):
                nParamsToSet = 3 + nPol
                
            for p in range(0, nParamsToSet ):
                fFitF.SetParameter(p,lastFitF.GetParameter(p));
                
            hist.Fit( fFitF, "RFQ" )
            lastFitF = fFitF
            fitOK = fFitF.GetParError(1) < m_maxErrorForOK 
            if fitOK:
                bestFitF = fFitF
                bestNPol = nPol
            else:
                if ( nPol == nPolFull ):
                    hist.Fit( fFitF, "RSE0Q" )
                    fitOK = True         
                if nPol > 1:
                    break 

        fitRes = [hist, fFitF.GetParameter(2), fFitF.GetParError(2)]
        return fitRes;



