from ROOT import TH1D, TCanvas, TFile, TF1, gStyle, TGraph, TMultiGraph, TLine, TH2D
import numpy

class CkResSummary:
    def __init__(self, _alignConf, _maxIt, ):
        self.alignConf = _alignConf
        self.nameStr = self.alignConf.getProp('nameStr')
        self.whichRich = self.alignConf.getProp('Rich')
        self.workdir = self.alignConf.getProp('WorkDir')
        self.maxIt = _maxIt # the maximum number of the iteration that we are looping through
        self.lastXMLFile = self.workdir + 'Rich' + str(self.whichRich) + 'CondDBUpdate_' + self.nameStr + '_i' + str(_maxIt + 1) + '.xml' 
        self.prename =  "RichRecQCHistos_rich" + str(self.whichRich) + "_" + self.nameStr + "_"
        self.fitMin = -0.007 # default: RICH1
        self.fitMax = 0.007 # default: RICH1
        if (self.whichRich == 2):
            self.fitMin = -0.0039
            self.fitMax = 0.0035
        self.maxpri = 3 # default: RICH1
        self.binToPriMirr = [[2,3],
                             [0,1]]
        self.maxsec = 15 # default: RICH1
        self.binToSecMirr = [[10,11,14,15],
                             [8,9,12,13],
                             [2,3,6,7],
                             [0,1,4,5]]
        if (self.whichRich == 2):
            self.maxpri = 55
            self.binToPriMirr = [[3,2,1,0,31,30,29,28],
                                 [7,6,5,4,35,34,33,32],
                                 [11,10,9,8,39,38,37,36],
                                 [15,14,13,12,43,42,41,40],
                                 [19,18,17,16,47,46,45,44],
                                 [23,22,21,20,51,50,49,48],
                                 [27,26,25,24,55,54,53,52]]
            self.maxsec = 39
            self.binToSecMirr = [[3,2,1,0,23,22,21,20],
                                 [7,6,5,4,27,26,25,24],
                                 [11,10,9,8,31,30,29,28],
                                 [15,14,13,12,35,34,33,32],
                                 [19,18,17,16,39,38,37,36]]
    
    def makeSummaryPlot(self, compareXML):
        import os
        gStyle.SetOptFit(1111)
        gStyle.SetOptStat(000000000)
        
        from tiltObj import tiltObj
        mirrtilts = tiltObj(self.alignConf)
        mirrtilts.setChange([compareXML, self.lastXMLFile])
        mgraphs = mirrtilts.getGraphs()
        
        if (self.whichRich == 1):
            heatHistoPriY = TH2D('heatPriY', 'RICH' + str(self.whichRich) + ' Primary mirrors local Y rotation (mrad)', 2, -1, 1, 2, -1, 1)
            heatHistoPriZ = TH2D('heatPriZ', 'RICH' + str(self.whichRich) + ' Primary mirrors local Z rotation (mrad)', 2, -1, 1, 2, -1, 1)
            heatHistoSecY = TH2D('heatSecY', 'RICH' + str(self.whichRich) + ' Secondary mirrors local Y rotation (mrad)', 4, -1, 1, 4, -1, 1)
            heatHistoSecZ = TH2D('heatSecZ', 'RICH' + str(self.whichRich) + ' Secondary mirrors local Z rotation (mrad)', 4, -1, 1, 4, -1, 1)
        if (self.whichRich == 2):
            heatHistoPriY = TH2D('heatPriY', 'RICH' + str(self.whichRich) + ' Primary mirrors local Y rotation (mrad)', 8, -1, 1, 7, -1, 1)
            heatHistoPriZ = TH2D('heatPriZ', 'RICH' + str(self.whichRich) + ' Primary mirrors local Z rotation (mrad)', 8, -1, 1, 7, -1, 1)
            heatHistoSecY = TH2D('heatSecY', 'RICH' + str(self.whichRich) + ' Secondary mirrors local Y rotation (mrad)', 8, -1, 1, 5, -1, 1)
            heatHistoSecZ = TH2D('heatSecZ', 'RICH' + str(self.whichRich) + ' Secondary mirrors local Z rotation (mrad)', 8, -1, 1, 5, -1, 1)
        
        resHistograms = {}    
        resHisto = TH1D('hres', 'RICH' + str(self.whichRich) + ' Cherenkov angle resolution per It. ', self.maxIt + 1, -0.5, self.maxIt + 0.5)
        
        saveName = self.workdir + 'Rich' + str(self.whichRich) + '_ckResSummary.pdf'
        cPlot = TCanvas('cPlot', 'ckRes')
        cPlot.SaveAs(saveName + '[')

        # heatHistos
        xTitle = 'relative global X position (not to scale)'
        yTitle = 'relative global Y position (not to scale)'
        heatHistoPriY.SetXTitle(xTitle)
        heatHistoPriY.SetYTitle(yTitle)     
        heatHistoSecY.SetXTitle(xTitle)
        heatHistoSecY.SetYTitle(yTitle)   
        heatHistoPriZ.SetXTitle(xTitle)
        heatHistoPriZ.SetYTitle(yTitle)     
        heatHistoSecZ.SetXTitle(xTitle) 
        heatHistoSecZ.SetYTitle(yTitle) 
        
        rowsPri = len(self.binToPriMirr)
        colsPri = len(self.binToPriMirr[0])
        for row in xrange(rowsPri):
            for col in xrange(colsPri):
                heatHistoPriY.SetBinContent(col+1, row+1, mirrtilts.priY[self.binToPriMirr[row][col]]) 
                heatHistoPriZ.SetBinContent(col+1, row+1, mirrtilts.priZ[self.binToPriMirr[row][col]])

        rowsSec = len(self.binToSecMirr)
        colsSec = len(self.binToSecMirr[0])
        for row in xrange(rowsSec):
            for col in xrange(colsSec):
                heatHistoSecY.SetBinContent(col+1, row+1, mirrtilts.secY[self.binToSecMirr[row][col]] )
                heatHistoSecZ.SetBinContent(col+1, row+1, mirrtilts.secZ[self.binToSecMirr[row][col]] )
        
        sTPY = float(self.alignConf.getProp('stopTolerancePriY'))
        sTPZ = float(self.alignConf.getProp('stopTolerancePriZ'))
        sTSY = float(self.alignConf.getProp('stopToleranceSecY'))
        sTSZ = float(self.alignConf.getProp('stopToleranceSecZ'))
        if int(self.alignConf.getProp('stopToleranceMode')) == 1:
            heatHistoPriY.GetZaxis().SetRangeUser(-10*sTPY, 10*sTPY)
            heatHistoPriZ.GetZaxis().SetRangeUser(-10*sTPZ, 10*sTPZ)
            heatHistoSecY.GetZaxis().SetRangeUser(-10*sTSY, 10*sTSY)
            heatHistoSecZ.GetZaxis().SetRangeUser(-10*sTSZ, 10*sTSZ)
        else:
            # for now this is the same as for stopToleranceMode == 1; however it is wrong to rely on the values of the stop tolerances in the configuration file, since those are not actually being used
            heatHistoPriY.GetZaxis().SetRangeUser(-10*sTPY, 10*sTPY) # Eventually we will have to extract these tolerances from RichMirrAlign
            heatHistoPriZ.GetZaxis().SetRangeUser(-10*sTPZ, 10*sTPZ) # Eventually we will have to extract these tolerances from RichMirrAlign
            heatHistoSecY.GetZaxis().SetRangeUser(-10*sTSY, 10*sTSY) # Eventually we will have to extract these tolerances from RichMirrAlign
            heatHistoSecZ.GetZaxis().SetRangeUser(-10*sTSZ, 10*sTSZ) # Eventually we will have to extract these tolerances from RichMirrAlign
        
        palette = numpy.array([636,635,634,633,632,807,797,800,400,416,416,432,867,857,600,616,617,618,619,620], dtype = 'intc')
        gStyle.SetPalette(20,palette);
        gStyle.SetPaintTextFormat('4.2f')
        
        heatHistoPriY.Draw("COLZ")
        heatHistoPriY.SetMarkerSize(2.0*heatHistoPriY.GetMarkerSize())
        heatHistoPriY.Draw("TEXT SAME")
        cPlot.SaveAs(saveName)
        heatHistoPriZ.Draw("COLZ")
        heatHistoPriZ.SetMarkerSize(2.0*heatHistoPriZ.GetMarkerSize())
        heatHistoPriZ.Draw("TEXT SAME")
        cPlot.SaveAs(saveName)
        heatHistoSecY.Draw("COLZ")
        heatHistoSecY.SetMarkerSize(2.0*heatHistoSecY.GetMarkerSize())
        heatHistoSecY.Draw("TEXT SAME")
        cPlot.SaveAs(saveName)
        heatHistoSecZ.Draw("COLZ")
        heatHistoSecZ.SetMarkerSize(2.0*heatHistoSecZ.GetMarkerSize())
        heatHistoSecZ.Draw("TEXT SAME")
        cPlot.SaveAs(saveName)
        
        # resHistos
        polbkg = TF1("polBKG","pol3(0)", self.fitMin, self.fitMax)
        help = None
        
        fitRes = []
        for j in range(0, self.maxIt + 1):
            filename = self.workdir + '/' + self.prename + 'i' +  str(j) + '.root'
            if os.path.exists(filename) :
                thisFile = TFile(filename, 'read')
                hist = thisFile.Get('RICH/RiCKResLongTightMirror/Rich' + str(self.whichRich) + 'Gas/ckResAll')
                title = 'RICH ' + str(self.whichRich) + ' Cherenkov angle resolution It. ' + str(j) 
                fitRes = self.fitCherenkovAngle(hist)
                resHistograms[j] = fitRes[0]                

                resHistograms[j].SetTitle(title)
                resHistograms[j].SetXTitle('#Delta#theta_{Cherenkov} (rad)')
                resHistograms[j].SetYTitle('Entries')
                help = resHistograms[j].GetFunction("Rich" + str(self.whichRich) + "fFitF3")
                polbkg.SetParameter(0, help.GetParameter(3))
                polbkg.SetParameter(1, help.GetParameter(4))
                polbkg.SetParameter(2, help.GetParameter(5))
                polbkg.SetParameter(3, help.GetParameter(6))
                polbkg.SetLineColor(4)

                resHistograms[j].Draw()
                polbkg.Draw('same')
                cPlot.SaveAs(saveName)
                resHisto.SetBinContent(j+1, fitRes[1])
                resHisto.SetBinError(j+1, fitRes[2])
        
        resHisto.SetBinContent(self.maxIt + 1, fitRes[1])
        resHisto.SetBinError(self.maxIt + 1, fitRes[2])
        resHisto.SetMarkerStyle(8)
        resHisto.SetXTitle('iteration number')
        resHisto.SetYTitle('Cherenkov angle resolution (rad)')
        resHisto.Draw()
        cPlot.SaveAs(saveName)

        #maxPriY = TLine(0, self.alignConf.getProp('stopTolerancePriY'), self.maxpri, self.alignConf.getProp('stopTolerancePriY'))
        #minPriY = TLine(0, -self.alignConf.getProp('stopTolerancePriY'), self.maxpri, -self.alignConf.getProp('stopTolerancePriY')) 
        #maxPriZ = TLine(0, self.alignConf.getProp('stopTolerancePriZ'), self.maxpri, self.alignConf.getProp('stopTolerancePriZ'))                                                                                
        #minPriZ = TLine(0, -self.alignConf.getProp('stopTolerancePriZ'), self.maxpri, -self.alignConf.getProp('stopTolerancePriZ'))    
        #maxSecY = TLine(0, self.alignConf.getProp('stopToleranceSecY'), self.maxpri, self.alignConf.getProp('stopToleranceSecY'))  
        #minSecY = TLine(0, -self.alignConf.getProp('stopToleranceSecY'), self.maxpri, -self.alignConf.getProp('stopToleranceSecY'))    
        #maxSecZ = TLine(0, self.alignConf.getProp('stopToleranceSecZ'), self.maxpri, self.alignConf.getProp('stopToleranceSecZ'))                                                                                
        #minSecZ = TLine(0, -self.alignConf.getProp('stopToleranceSecZ'), self.maxpri, -self.alignConf.getProp('stopToleranceSecY'))    
        #maxPriY.SetLineColor(2)
        #minPriY.SetLineColor(2)
        #maxPriZ.SetLineColor(4)
        #minPriZ.SetLineColor(4)
        #maxSecY.SetLineColor(2)
        #minSecY.SetLineColor(2)
        #maxSecZ.SetLineColor(4)
        #minSecZ.SetLineColor(4)
        
        # compensations graphs
        mgraphs[0].Draw('AP')
        #maxPriY.Draw('same')
        #minPriY.Draw('same')
        #maxPriZ.Draw('same')
        #minPriZ.Draw('same')
        mgraphs[0].SetTitle("RICH" + str(self.whichRich) +" primary mirror compensations w.r.t. to comparison DB")
        mgraphs[0].GetXaxis().SetTitle("primary mirror number")
        mgraphs[0].GetYaxis().SetTitle("change in mirror compensation (mrad) [red: local Y, blue: local Z]")
        cPlot.SaveAs(saveName)
        mgraphs[1].Draw('AP')
        #maxSecY.Draw('same')
        #minSecY.Draw('same')
        #maxSecZ.Draw('same')
        #minSecZ.Draw('same')
        mgraphs[1].SetTitle("RICH" + str(self.whichRich) +" secondary mirror compensations w.r.t. to comparison DB")
        mgraphs[1].GetXaxis().SetTitle("secondary mirror number")
        mgraphs[1].GetYaxis().SetTitle("change in mirror compensation (mrad) [red: local Y, blue: local Z]")
        cPlot.SaveAs(saveName)
        
        cPlot.SaveAs(saveName + ']')
        
        mirrtilts.writeChange()
        mgraphs[0].Delete()
        mgraphs[1].Delete()

    def fitCherenkovAngle(self, hist):
        richStr = 'Rich' + str(self.whichRich)
        m_maxErrorForOK = 1e-3;
        
        xPeak = hist.GetBinCenter( hist.GetMaximumBin() )
        delta = 0
        if (richStr == 'Rich1'):
            delta = 0.0025
        else:
            delta = 0.00105
        
        fitMin = xPeak - delta
        fitMax = xPeak + delta
        
        preFitFName = richStr + "PreFitF"
        preFitF = TF1( preFitFName, "gaus", fitMin, fitMax )
        preFitF.SetParameter(1, 0 )
        
        if (richStr == 'Rich1') :
            preFitF.SetParameter(2, 0.0015 )
        else:
            preFitF.SetParameter(2, 0.0007 )
        
        hist.Fit(preFitF, "RQ" );
        fitOK = False;
        
        lastFitF = preFitF
        bestFitF = None
        nPolFull = 3
        bestNPol = 0
        
        fFitF = None

        for nPol in range(1, nPolFull + 1):
            fFitFName = richStr + "fFitF" + str(nPol)
            fFitFType = "gaus(0)+pol"+ str(nPol)+"(3)" 
            fFitF = TF1( fFitFName, fFitFType, self.fitMin, self.fitMax )
            
            fFitF.SetParName(0,"Gaus Constant")
            fFitF.SetParName(1,"Gaus Mean")
            fFitF.SetParName(2,"Gaus Sigma")
            nParamsToSet = 3 
            if ( nPol > 1 ):
                nParamsToSet = 3 + nPol
                
            for p in range(0, nParamsToSet ):
                fFitF.SetParameter(p,lastFitF.GetParameter(p));
                
            hist.Fit( fFitF, "RFQ" )
            lastFitF = fFitF
            fitOK = fFitF.GetParError(1) < m_maxErrorForOK 
            if fitOK:
                bestFitF = fFitF
                bestNPol = nPol
            else:
                if ( nPol == nPolFull ):
                    hist.Fit( fFitF, "RSE0Q" )
                    fitOK = True         
                if nPol > 1:
                    break 

        fitRes = [hist, fFitF.GetParameter(2), fFitF.GetParError(2)]
        return fitRes;



