from Communicator import *
import distutils
import os
import re
import shutil
import sys
import math
import time
from   time       import sleep
from   time       import time
from   time       import gmtime, strftime
import threading
import Configurables
from   subprocess import *
from   distutils  import dir_util

print "INFO: Iterator.py launched at " + strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC"
print 'INFO: Claire says miezmiezmiez'
sys.stdout.flush()


########################################################################################################################                                    
##### Here is where the party starts!
########################################################################################################################     
def run(whichRich):
  launchTime = strftime("%Y-%m-%d %H:%M:%S", gmtime()) 
  start_time_config = time() #  when LHCbA config start time is really set

  ### Start logging of hlt02 (the node that the iteraror is running on) 
  loggingDir = '/group/rich/AlignmentFiles/Logging/'
  cmdProcess = 'exec /opt/FMC/bin/logViewer -l 2 -S -b &> ' + loggingDir + 'Rich' + str(whichRich) + '_' + 'hlt02.log'
  logProcess = Popen(cmdProcess, shell=True, executable="/bin/bash", stderr=PIPE, stdout=PIPE)  
  logProcess.stdout.close()
  logProcess.stderr.close()
  sys.stdout.flush()    
  sleep(8) # wait long enough for logger to initialize

  ### Printout (slightly late, so it gets logged)
  print "INFO: run(whichRich) command received at " + launchTime + " UTC"
  print "INFO: LHCbA config start time set."
  print "INFO: logging hlt02 in temporary file: " + loggingDir + 'Rich' + str(whichRich) + '_' + 'hlt02.log'
  sys.stdout.flush()

  ### make a backup of OnlineEnvBase.py (do it now, in case we need to study the configuration on a single node)
  shutil.copy2('/group/online/dataflow/options/LHCbA/HLT/OnlineEnvBase.py' , loggingDir + 'Rich' + str(whichRich) + '_' + 'OnlineEnvBase.py' )
  print "INFO: storing OnlineEnvBase.py in temporary file: " + loggingDir + 'Rich' + str(whichRich) + '_' + 'OnlineEnvBase.py'
  sys.stdout.flush()
  
  ### make a backup of Configuration.py (make sure AlignmentRelease on /group/rich/sw/cmtuser/ points to our newest AlignmentOnline release!)
  shutil.copy2('/group/rich/sw/cmtuser/AlignmentRelease/Rich/RichMirrorAlignmentOnline/python/RichMirrorAlignmentOnline/Configuration.py' , loggingDir + 'Rich' + str(whichRich) + '_' + 'Configuration.py' )
  print "INFO: storing Configuration.py in temporary file: " + loggingDir + 'Rich' + str(whichRich) + '_' + 'Configuration.py'
  sys.stdout.flush()

  from HistoHelper import SaveSetRetriever
  ssRetr = None  #the SavesetRetriever, this thing will give us the path to the histogram files produced by the analyzers
  from HistoHelper import HistoCopier
  histCopier = None  #the Histcopier, it will wait after each minor iteration for the savesets to be written and copy it under the approriate name into workdir
  from XMLFileHelper import XMLFileHelper
  xmlHelper = None  
  from RichAlignmentHelper import RichAlignmentHelper
  alignHelper = None
  from SetupHelper import SetupHelper
  setupHelper = None
  from UpdateHelper import UpdateHelper
  updateHelper = None
  
  ########################################################################################################################
  #####  reading in the settings from the Configuration file
  ########################################################################################################################
  print "INFO: Importing Configuration"
  sys.stdout.flush()
  if whichRich == 1:
    from Configurables import Rich1MirrAlignOnConf
    alignConf = Rich1MirrAlignOnConf()
  elif whichRich == 2:
    from Configurables import Rich2MirrAlignOnConf
    alignConf = Rich2MirrAlignOnConf()
  else:
    sys.stderr.write("A detector named RICH"+ str(whichRich) +" does not exist.\n")
    sys.exit(1)
  alignConf.__apply_configuration__()
  print "INFO: Configuration Applied"
  sys.stdout.flush()
    
  maxIters = int(alignConf.getProp('maxIterations')) # This is the maximum number of iterations before the alignment gives up
    
  thisCase = alignConf.getProp('nameStr')
  workdir = alignConf.getProp('WorkDir')
  histdir = alignConf.getProp('HistoDir')
  tiltNames = alignConf.getProp('tiltNames')
  tiltNamesLength = len(tiltNames)
  thisNameStr = alignConf.getProp('nameStr')
  magnFactorsMode = alignConf.getProp('magnFactorsMode')
  autoUpdate = bool(alignConf.getProp('autoUpdate'))
  testing = bool(alignConf.getProp('testing'))
  print "INFO: Initial setting of autoUpdate = " + str(autoUpdate)
  print "INFO: Initial setting of testing = " + str(testing)
  
  combAndMirrSubsets = workdir + "Rich" + str(whichRich) + "CombAndMirrSubsets.txt"
  
  ### this is the place where the analyzers will pick up the xml file for their Brunel jobs
  ### before each new minor iteration the new xml file will be copied here
  currentXML = workdir + "CondDB_Rich" + str(whichRich) + ".xml"

  ### path in the workdirectory of the database-file that the final database will be compared to  
  compareXML = workdir + 'Rich' + str(whichRich) + '_compareXMLFile.xml'
  
  # We do this before a "configure" because we need run and directoryTime info for the next part
  setupHelper = SetupHelper(alignConf) 

  # Do not run the mirror alignment if: 
  #   Any run's polarity is OFF 
  #   The polarities for the runs chosen are mixed
  import OnlineEnv as Online
  runs = Online.DeferredRuns
  polarities = []
  fills = []
  for run in runs:
      polarities.append(setupHelper.getPolarity(run))
      fills.append(setupHelper.getFillNumber(run))
  print "INFO: runs = " + str(runs)
  print "INFO: fills = " + str(fills)
  polsStr = [str(item) for item in polarities]
  print "INFO: polarities = " + str(polsStr) # gets rid of unicode flag preceding each string
  cancel = False
  preSubject, preText, postSubject, postText = [''] * 4 # this is a nice variable defining strategy, but note it does not work on mutable elements (such as lists)!
  for i in range(0, len(runs)):
    if (polarities[i] is None):
      preSubject += 'Polarity information missing, but '
      preText += 'Polarity information for at least one of the runs is currently not found. \n'      
      print 'WARNING: Polarity information for at least one of these runs is currently not found: ' + str(runs) + '. This alignment will be started shortly.'
      break
    if (polarities[i] == 'OFF') or (polarities[i] != polarities[0]):
      cancel = True
      setupHelper.sendEmail('RICH' + str(whichRich) + ' mirror alignment started, but skipped', 'Some of the chosen runs contain magnet OFF data *or* Data for these runs is of mixed polarity: \n Runs: ' + str(runs) + ' \n Fills: ' + str(fills) + ' \n Polarities: ' + str(polsStr) , setupHelper.emails)
      print 'INFO: Mirror alignment will be skipped. Some of these runs contain magnet OFF data *or* Data for these runs is of mixed polarity: ' + str(runs)
      break      
  polarityDir = '/group/rich/AlignmentFiles/Polarity/'
  forcedUpdate_ChMagPol = False
  if (not cancel) and (polarities[0] is not None):
    oldPolarityFilename = polarityDir + 'previousPolarity_RICH' + str(whichRich) +'.txt'
    newPolarityFilename = polarityDir + 'currentPolarity_RICH' + str(whichRich) +'.txt'
    with open(newPolarityFilename, 'w') as currentAlignPolarityFile:
      currentAlignPolarityFile.write(polarities[0] + '\n' + str(runs) + '\n' + str(fills) + '\n')        
    if os.path.exists(oldPolarityFilename):
      with open(oldPolarityFilename, 'r') as previousAlignPolarityFile:
        first_line = previousAlignPolarityFile.readline()
      if (first_line != polarities[0] + '\n'):
        forcedUpdate_ChMagPol = True
        postSubject += ' Magnet Polarity Changed; possibly forcing update for RICH' + str(whichRich) + ' alignment ' + setupHelper.directoryTime
        postText += ' \n If RICH' + str(whichRich) + ' alignment ' + setupHelper.directoryTime + ' converges, the alignment will be updated due to a change in magnet polarity (unless autoUpdate is/becomes False, or testing is True).'
        print 'INFO: Magnet Polarity Changed; if this alignment converges, the alignment will be updated.'
    shutil.copy2(newPolarityFilename, oldPolarityFilename) 

  if (not cancel):
    leadText = ''
    if testing is True:
      leadText += 'TEST [' + alignConf.getProp('dataVariant') + ']: '
    setupHelper.sendEmail( leadText + preSubject + 'RICH' + str(whichRich) + ' alignment ' + setupHelper.directoryTime + ' has started.' + postSubject , preText + ' RICH' + str(whichRich) + ' mirror alignment ' + setupHelper.directoryTime + ' has started. \n Runs: ' + str(runs) + ' \n Fills: ' + str(fills) + ' \n Polarities: ' + str(polsStr) + postText + ' \n dataVariant: ' + alignConf.getProp('dataVariant'), setupHelper.emails)        
    print 'INFO: RICH' + str(whichRich) + ' mirror alignment ' + setupHelper.directoryTime + ' has started, dataVariant: ' + alignConf.getProp('dataVariant')

  sys.stdout.flush()

  ########################################################################################################################  
  ##### define the communicator who will communicate with the run-control
  ########################################################################################################################
  
  state = State.NOT_READY
  if not cancel:
    com = Communicator('ALIGNITER')
    com.set_status(state)
  
  conv = False  ### variable indicates if alignment converged yet

  while not cancel:
  #while True:
    sys.stdout.flush() # Always keep this line in
    print "INFO: Waiting for command from Communicator"
    sys.stdout.flush()  
    command = com.get_command()
    ########################################################################################################################                                
    ##### this code will be excecuted when you click configure on the run-control
    ########################################################################################################################       
    if command.startswith('configure') and state == State.NOT_READY:
      print "INFO: CONFIGURE command received at " + strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC"
      sys.stdout.flush() 
      xmlHelper = XMLFileHelper(alignConf)
      ssretr = SaveSetRetriever("LHCbA", "AligWrk_Rich" + str(whichRich))
      histCopier = HistoCopier(alignConf, ssretr)
      alignHelper = RichAlignmentHelper(alignConf, combAndMirrSubsets)
      updateHelper = UpdateHelper(alignConf)  
      ### start the Mirror Alignment DB update DIM server for this RICH detector
      if testing is not True:      
          updateHelper.startUpdateDIM()    
      else:
          print 'INFO: The alignment is in testing mode. DB update DIM service disabled.'
      sys.stdout.flush()  
      ### get the name of the xml file that is equivalent to "the DB alignment" within the '/group/online/alignment/' directory on plus
      latestName = xmlHelper.getStartXMLOrigFileName()
      latest_vN = (latestName.replace(".xml", "")).replace("v", "")
      state = State.READY # Though LHCbA is still configuring
      sys.stdout.flush()  
      
    ########################################################################################################################                                
    ##### setting up the alignment and starting the first minor iteration (this only for the very first iteration)           
    ######################################################################################################################## 
    elif (command.startswith('start') and state == State.READY):
      print "INFO: START command received at " + strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC"
      sys.stdout.flush()  

      ########################################################################################################################
      ##### iteration counters, n_it for major iteration, m_it for minor iteration
      ########################################################################################################################
      n_it = alignConf.getProp("MajItStart")
      m_it = alignConf.getProp("MinItStart")
      if (n_it != 0) or (m_it != 0):
        print "INFO: n_it, m_it =", n_it, ",", m_it

      try:
        elapsed_time_config
      except NameError:
        elapsed_time_config = time() - start_time_config # LHCbA configuration time
      else:
        elapsed_time_config = 0.0 # we were already configured
        if (n_it == 0) and (m_it == 0):
            setupHelper.directoryTime = strftime("%Y%m%d_%H%M%S", gmtime()) # create a new storage directory if we STARTed a second clean alignment from the same Iterator configuration
      print "INFO: Elapsed time to configure LHCbA set as", elapsed_time_config, "seconds." # only accurate if LHCbA AutoPilot is ON

      elapsed_runtimes = {'elapsed_time_run' : 0.0, 'elapsed_times_RichMirrCombinFit' : [], 'elapsed_times_RichMirrAlign' : []}
      start_time_run = time() 
      print "INFO: Alignment start time set."
      sys.stdout.flush()  

      ### write the file for the analyzers to know which iteration they are at
      f = open(alignConf.getProp("ItNrFile"), "w")
      f.write(str(m_it))
      f.close()
      t = time()
      
      ### if you want to start the alignment from scratch (e.g. at the m_it = 0) 
      if m_it == 0:
        #clean up the working directory
        print "INFO: Setting up the folders and getting the starting XML"
        print "INFO: Removing everything in " + workdir
        for f in os.listdir(workdir):
          if os.path.isdir(workdir + "/" + f):
            shutil.rmtree(workdir + "/" + f)
          else:
            os.remove(workdir + "/" + f)
            
        shutil.copy2("/group/rich/AlignmentFiles/Rich" + str(whichRich) + "CombAndMirrSubsets.txt", combAndMirrSubsets)
        currentMirrorXMLFile = workdir + "Rich" + str(whichRich) + "CondDBUpdate_" + thisNameStr + "_i" + str(n_it) + ".xml"
        
        xmlHelper.getStartXML(currentMirrorXMLFile)   ### get the starting-xml from the database
        xmlHelper.getCompareXML(compareXML) ### can be different from starting-xml
        if not magnFactorsMode == 0:
          xmlHelper.create_coeff_calibration_xml_files(currentMirrorXMLFile, n_it)    ### create the xml files for the tilted mirror configurations 
          sys.stdout.flush()
        
      ### if you want to start from some arbitrary minor iteration
      else:
        #this method will keep all files needed when starting at m_it > 0
        setupHelper.startMinIt(m_it)
        
      ### now place the xml file for the current minor iteration in the place where the analyzers will pick it up
      if m_it % tiltNamesLength == 0:
        pickupXML = workdir + "Rich" + str(whichRich) + "CondDBUpdate_" + thisNameStr + "_i" + str(n_it) + ".xml"
      else:
        pickupXML = workdir + "Rich" + str(whichRich) + "CondDBUpdate_" + thisNameStr + "_" + tiltNames[m_it % tiltNamesLength] + "_i" + str(n_it) + ".xml"
      shutil.copy2(pickupXML, currentXML)
 
      print "INFO:    ___________________________ "      
      print "INFO:    Start new major Iteration " + str(n_it)
      print "INFO:    *************************** " 
      
      print "INFO:    Minor iteration " + str(m_it)
      
      # increate the minor iteration counter
      if magnFactorsMode == 0:
        setupHelper.setupMagnifFiles()
        m_it = m_it + tiltNamesLength
      else:
        m_it = m_it + 1
      # set iterator state to running => analyzers will exectute the Brunel jobs now 
      state = State.RUNNING
      sys.stdout.flush()  

    ######################################################################################################################## 
    ##### here the code for all the following iterations
    ########################################################################################################################  
    elif (command.startswith('pause') and state == State.RUNNING):
      print "INFO: PAUSE command received at " + strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC"
      state = State.PAUSED
      com.set_status(state)
      
      # write new minor iteration number to file for the analyzers to read
      f = open(alignConf.getProp("ItNrFile"), "w")
      f.write(str(m_it))
      f.close()
      
      # copy the saveset made by the analyzers in the last iteration to the desired place in workdir
      if magnFactorsMode == 0:
        histCopier.getHistoFile(n_it, m_it - tiltNamesLength);
      else:
        histCopier.getHistoFile(n_it, m_it - 1);
        
      sys.stdout.flush()  
      
      # if the minor iterations for one major iteration have finished perform fits etc: 
      if(m_it % tiltNamesLength == 0):
        print "INFO:    minor iterations for major iteration " + str(n_it) + " finished"
        print "INFO:    now merge the rootfiles and perform alignment"        
        
        ### run the mirror Alignment
        start_time_RichMirrCombinFit = time()
        alignHelper.runMirrCombinFit(n_it)
        hh, r = divmod(time() - start_time_RichMirrCombinFit, 3600)
        mm, ss = divmod(r, 60)
        elapsed_runtimes['elapsed_times_RichMirrCombinFit'].append("{:0>2}:{:0>2}:{:05.2f}".format(int(hh),int(mm),ss)) # RichMirrCombinFit run time
        
        alignHelper.checkFitQual(n_it)

        start_time_RichMirrAlign = time()
        alignHelper.runRichAlign(n_it)
        hh, r = divmod(time() - start_time_RichMirrAlign, 3600)
        mm, ss = divmod(r, 60)
        elapsed_runtimes['elapsed_times_RichMirrAlign'].append("{:0>2}:{:0>2}:{:05.2f}".format(int(hh),int(mm),ss)) # RichMirrAlign run time
        
        ### pick up the new xml file and format in in a way readable for the analyzers
        newXMLfile = workdir + "Rich" + str(whichRich) + "CondDBUpdate_" + thisNameStr + "_i" + str(n_it + 1) + ".xml"            
        xmlHelper.reformatXML(newXMLfile, workdir)
        
        ### check if the alignment has converged or failed
        conv = alignHelper.hasConverged()
        print "INFO: The alignment after Iteration", n_it, "returns converged =", conv
        
        allDone = False
        if(conv or n_it >= maxIters):
          allDone = True
          if conv and n_it >= maxIters:
            print "INFO: *** The alignment has reached its maximum number of iterations and has converged after Iteration", n_it, "***"
          elif conv:
            print "INFO: *** The alignment has converged after Iteration", n_it, "***"
          else:
            print "INFO: *** The alignment has reached its maximum number of iterations and has not converged ***"

          elapsed_runtimes['elapsed_time_run'] = time() - start_time_run # Actual total iterating time
          print "INFO: Elapsed time to run the Mirror Alignment set as", elapsed_runtimes['elapsed_time_run'], "seconds."

          nEventsListList = setupHelper.getNEventsListList(n_it)
          if nEventsListList[1] < int(alignConf.getProp('requiredEvents')):
              if autoUpdate is True:
                autoUpdate = False
                print "INFO: We processed an average number of events per iteration of " + str(nEventsListList[1]) + ", which is too low to allow mirror alignment " + setupHelper.directoryTime + " to automatically update, should an update be deemed necessary."
                setupHelper.sendEmail( 'autoUpdate disabled for RICH' + str(whichRich) + ' alignment ' + setupHelper.directoryTime + '.' , 'RICH' + str(whichRich) + ' mirror alignment ' + setupHelper.directoryTime + ' will not automatically be updated, should an update be deemed necessary. \n Average number of events processed per iteration: ' + str(nEventsListList[1]) , setupHelper.emails)        

          sys.stdout.flush()  
          
          if forcedUpdate_ChMagPol and conv:
              # if we are forcing the update we use the newXML from RichMirrAlign, not the currentXML
              shutil.copy2(newXMLfile,currentXML) 
          if (forcedUpdate_ChMagPol or updateHelper.updateDecision(n_it)) and conv:
              if autoUpdate:
                xmlHelper.finalXML( str(int(latest_vN) + 1) , True) # current + 1
                setupHelper.finalize(conv, n_it, compareXML, elapsed_time_config, elapsed_runtimes, True, nEventsListList)
                if testing is not True:
                    updateHelper.updateDB( int(latest_vN) + 1 , True) # current + 1
              else:
                xmlHelper.finalXML( str(int(latest_vN) + 1) + '_maybe_' + setupHelper.directoryTime, True) # (current + 1)_maybe
                setupHelper.finalize(conv, n_it, compareXML, elapsed_time_config, elapsed_runtimes, None, nEventsListList)
                if testing is not True:
                    updateHelper.updateDB( int(latest_vN) , False)
              #updateHelper.stopUpdateDIM()    
          else:
              setupHelper.finalize(conv, n_it, compareXML, elapsed_time_config, elapsed_runtimes, False, nEventsListList)
              if testing is not True:
                  updateHelper.updateDB( int(latest_vN) , False) # current 
                  #updateHelper.stopUpdateDIM()  
          state = State.READY
          
        ### if the alignment has not converged and the maximal number of iterations has not been reached,
        ### set up everything for the next major iteration
        if n_it < maxIters and not conv:
          n_it = n_it + 1
          print "INFO:    ___________________________ "      
          print "INFO:    Start new major Iteration " + str(n_it)
          print "INFO:    *************************** " 
          ### get the xml-file produced by the last fit and create the ones with the tiled mirrors
          currentMirrorXMLFile = workdir + "Rich" + str(whichRich) + "CondDBUpdate_" + thisNameStr + "_i" + str(n_it) + ".xml"
          if not magnFactorsMode == 0:
            xmlHelper.create_coeff_calibration_xml_files(currentMirrorXMLFile, n_it)
            sys.stdout.flush()
              
      ### just start the next minor iteration if not converged yet
      if n_it <= maxIters and not conv and not allDone:
        print "INFO: Major iteration " + str(n_it)
        print "INFO:    Minor iteration " + str(m_it)
        if(m_it % tiltNamesLength == 0):
          pickupXML = workdir + "Rich" + str(whichRich) + "CondDBUpdate_" + thisNameStr + "_i" + str(n_it) + ".xml"
        else:
          pickupXML = workdir + "Rich" + str(whichRich) + "CondDBUpdate_" + thisNameStr + "_" + tiltNames[m_it % tiltNamesLength] + "_i" + str(n_it) + ".xml"
          
        shutil.copy2(pickupXML, currentXML)
        if magnFactorsMode == 0:
          m_it = m_it + tiltNamesLength
        else:
          m_it = m_it + 1
        state = State.RUNNING
      sleep(2)
      
    elif command.startswith('stop') and state in (State.RUNNING, State.READY):
      print "INFO: STOP command received at " + strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC"
      elapsed_runtimes['elapsed_time_run'] = time() - start_time_run # Actual iterating time
      print "INFO: Elapsed time to run the Mirror Alignment set as", elapsed_runtimes['elapsed_time_run'], "seconds."
      nEventsListList = setupHelper.getNEventsListList(n_it)
      setupHelper.finalize(conv, n_it, compareXML, elapsed_time_config, elapsed_runtimes, False, nEventsListList)
      if testing is not True:
          updateHelper.updateDB( int(latest_vN) , False) # current
          #updateHelper.stopUpdateDIM()   
      state = State.READY
      sys.stdout.flush()  

    elif command.startswith('reset'):
      print "INFO: RESET command received at " + strftime("%Y-%m-%d %H:%M:%S", gmtime()) + " UTC"
      sys.stdout.flush()  
      state = State.NOT_READY
      break

    else:
      print 'ERROR: iterator: bad transition from %s to %s' % (state, command)
      sys.stdout.flush()  
      state = State.ERROR
      break
    # Set our status
    com.set_status(state)
  # Set our status one last time
  if not cancel:
    com.set_status(state)
  else:
    print "INFO: We choose to not execute the mirror alignment. "
    print "INFO: The input data is of mixed polarity, or some it is magnet OFF data."
    
########################################################################################################################                                    
##### This is where the party ends!
########################################################################################################################   
    
  print "INFO: End of run(whichRich)."
  sys.stdout.flush()
  
  if not cancel:
    try:
      shutil.copy2(loggingDir + 'Rich' + str(whichRich) + '_' + 'Configuration.py', setupHelper.savedir + 'Configuration.py')  
      print "INFO: Configuration.py stored in file: " + setupHelper.savedir + 'Configuration.py'
    except:
      print "INFO: Problem moving Configuration.py. Please try copying manually from /group/rich/AlignmentFiles/Logging/."
    try:
      shutil.copy2(loggingDir + 'Rich' + str(whichRich) + '_' + 'OnlineEnvBase.py', setupHelper.savedir + 'OnlineEnvBase.py')  
      print "INFO: OnlineEnvBase.py stored in file: " + setupHelper.savedir + 'OnlineEnvBase.py'
    except:
      print "INFO: Problem moving OnlineEnvBase.py. Please try copying manually from /group/rich/AlignmentFiles/Logging/."
    sys.stdout.flush()

  try:
    logProcess.kill()
    print "INFO: hlt02 logProcess killed."
  except OSError:
    # can't kill a dead process
    print "INFO: hlt02 logProcess already dead."
  sys.stdout.flush()
    
  if not cancel:  
    try:
      print "INFO: logging hlt02 ended, storing in file: " + setupHelper.savedir + 'hlt02.log'
      shutil.copy2(loggingDir + 'Rich' + str(whichRich) + '_' + 'hlt02.log', setupHelper.savedir + 'hlt02.log')  
    except:
      print "INFO: Problem moving hlt02.log. Please try copying manually from /group/rich/AlignmentFiles/Logging/."
    sys.stdout.flush()  
  
  sleep(8)
   
if __name__ == '__main__':
    run()


