__version__ = "$Id: BrunelOnline.py,v 1.25 2010/11/09 12:20:55 frankb Exp $"
__author__  = "Markus Frank <Markus.Frank@cern.ch>"

import os, sys
# to use the git CondDB
os.environ['GITCONDDBPATH'] = '/cvmfs/lhcb.cern.ch/lib/lhcb/git-conddb.test'
# Paras added these next 3
import re, importlib, __builtin__
import Configurables as Configs
import Gaudi.Configuration as Gaudi
from Gaudi.Configuration import appendPostConfigAction
import GaudiKernel
from GaudiKernel.SystemOfUnits import GeV, MeV
from GaudiKernel.ProcessJobOptions import PrintOff,InstallRootLoggingHandler,logging
from Configurables import CondDB, GaudiSequencer, EventPersistencySvc, HistogramPersistencySvc, EventLoopMgr, OutputStream, Gaudi__SerializeCnvSvc, DstConf
from Communicator import *
import time
#PrintOff(999)
InstallRootLoggingHandler(level=logging.CRITICAL)

processingType ='DataTaking'

GaudiKernel.ProcessJobOptions._parser._parse_units(os.path.expandvars("$STDOPTS/units.opts"))
requirement = None

debug = 0
def dummy(*args,**kwd): pass

MSG_VERBOSE = 1
MSG_DEBUG   = 2
MSG_INFO    = 3
MSG_WARNING = 4
MSG_ERROR   = 5
MSG_FATAL   = 6
MSG_ALWAYS  = 7

class saveHistograms(object):
  def __init__(self, runType, saveDir, n):
    self.__runType = runType
    self.__saveDir = saveDir
    self.__n = n
    
  def __call__(self):
    from Configurables import GaudiSequencer#, UpdateAndReset
    app=Gaudi.ApplicationMgr()
    app.ExtSvc += ['MonitorSvc']
    
    from Configurables import MonitorSvc
    MonitorSvc().OutputLevel = MSG_ERROR
    MonitorSvc().UniqueServiceNames = 1
    MonitorSvc().CounterUpdateInterval = -1
    MonitorSvc().DimUpdateInterval = -1
    Online = importOnline()
    MonitorSvc().PartitionName         = Online.PartitionName
    MonitorSvc().ProgramName           = "AligWork_%s_%02d" % (self.__runType, self.__n)
    
def staticVar(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

@staticVar("Online", None)
def importOnline():
  if importOnline.Online:
    print 'INFO: importOnline.Online = ', importOnline.Online
    return importOnline.Online
  if 'RUNINFO' in os.environ:
    runinfo = os.environ['RUNINFO']
    d, f = os.path.split(runinfo)
    name, ext = os.path.splitext(f)
    sys.path.insert(1, d)
    Online = importlib.import_module('OnlineEnv')
    sys.path.remove(d)
  else:
    import OnlineEnv as Online
  importOnline.Online = Online
  return Online

#===============================================================================
class ChangeP(object):
    def __init__(self, minP):
        self.__minP = minP

    def __call__(self):
        from Configurables import  PatForward, PatForwardTool
        for fwd in ("PatForwardHLT1", "PatForwardHLT2"):
            PatForward(fwd).addTool(PatForwardTool)
            PatForward(fwd).PatForwardTool.MinMomentum = self.__minP

#===============================================================================
def patchBrunel(true_online_version, alignConf, n):
  whichRich = alignConf.getProp('Rich')
  print "INFO: (RichAnalyzer.py) whichRich = ", whichRich
  workdir = alignConf.getProp('WorkDir')
  print "INFO: (RichAnalyzer.py) workdir = ", workdir
  whichData = alignConf.getProp('dataVariant')
  print "INFO: (RichAnalyzer.py) whichData = ", whichData

  import GaudiConf.DstConf
  
  # Brunel Configuration
  
  import Brunel.Configuration
  Online = importOnline()

  brunel = Brunel.Configuration.Brunel()
  brunel.Hlt1FilterCode = "HLT_PASS_RE('Hlt1CalibRICHMirror(RICH%s)?Decision')"%whichRich

  brunel.RichSequences = ["RICH"] # Required for Brunel v52r2 and above, which now uses the RichFuture framework by default. This means we are still on the old framework and must upgrade. 

  from Configurables import RecMoniConf
  brunel.RecoSequence = ['Decoding', "VELO", "TT", "IT", "OT", "TrHLT1","Vertex", "TrHLT2", 'RICH']
  RecMoniConf ().MoniSequence = ["RICH"]

  brunel.Detectors = ['Velo', 'PuVeto', 'Rich1', 'Rich2', 'TT', 'IT', 'OT', 'Magnet', 'Tr']

  import ConditionsMap
  brunel.OnlineMode = True
  brunel.DDDBtag    = ConditionsMap.DDDBTag
  print "INFO: (RichAnalyzer.py) brunel.DDDBtag = ", brunel.DDDBtag

  brunel.CondDBtag  = ConditionsMap.CondDBTag
  if ('Collision16' in whichData) or ('Collision15' in whichData):
      brunel.CondDBtag = "cond-20161011"
  print "INFO: (RichAnalyzer.py) brunel.CondDBtag = ", brunel.CondDBtag

  brunel.Histograms =  "Expert"

  brunel.DataType = "2018"
  if 'Collision17' in whichData:
      brunel.DataType = "2017"
  if 'Collision16' in whichData:
      brunel.DataType = "2016"
  if 'Collision15' in whichData:
      brunel.DataType = "2015"

  brunel.UseDBSnapshot = True

  brunel.OutputType='NONE'

  brunel.PartitionName = os.environ.get('PARTITION_NAME', 'TEST')
  print "INFO: (RichAnalyzer.py) brunel.PartitionName = ", brunel.PartitionName

  # CondDB Configuration

  from Configurables import CondDB
  conddb = CondDB()
  conddb.Online = True
  conddb.EnableRunChangeHandler = True

  conddb.UseDBSnapshot = False
  if ('Collision16' in whichData) or ('Collision15' in whichData):
      conddb.UseDBSnapshot = True

  conddb.EnableRunStampCheck = False
  #   conddb.EnableRunStampCheck = True
      
  if not ( ('Collision16' in whichData) or ('Collision15' in whichData) ):
      conddb.Tags['ONLINE'] = Online.OnlDBTag #needed when using gitDB
      print "INFO: (RichAnalyzer.py) conddb.Tags['ONLINE'] = ", conddb.Tags['ONLINE']

  # RunChangeHandler Configuration

  # Dictionary of the main mirror alignment conditions
  mirrAlignConds = {
    'Rich1' : [
              'Conditions/Alignment/Rich1/SecMirrorMasterTop_Align' , # not sure if this is really a main mirror alignment condition but for some reason is in this grouping
              'Conditions/Alignment/Rich1/SecMirrorMasterBot_Align'   # not sure if this is really a main mirror alignment condition but for some reason is in this grouping
              ]
              + ['Conditions/Alignment/Rich1/SphMirror%d_Align' %i for i in range(0, 4)]
              + ['Conditions/Alignment/Rich1/SecMirror%d_Align' %i for i in range(0, 16)],
    'Rich2' : []
              + ['Conditions/Alignment/Rich2/Rich2SphMirrorCont%d_Align' % i for i in range (0,2)]
              + ['Conditions/Alignment/Rich2/Rich2SecMirrorCont%d_Align' % i for i in range (0,2)]
              + ['Conditions/Alignment/Rich2/Rich2SphMirror%d_Align' % i for i in range(0, 56)]
              + ['Conditions/Alignment/Rich2/Rich2SecMirror%d_Align' % i for i in range(0, 40)]
    }  
  #print "INFO: (RichAnalyzer.py) mirrAlignConds = ", mirrAlignConds

  # Dictionary of the other conditions that show up in our XML 
  hpdConds = {
    'Rich1' : ['Conditions/Alignment/Rich1/Rich1_Align']
              + ['Conditions/Alignment/Rich1/HPD%d_Align' %i for i in range(0, 196)]
              + ['Conditions/Alignment/Rich1/HPDPanel%d_Align' %i for i in range(0, 2)],
    'Rich2' : ['Conditions/Alignment/Rich2/Rich2_Align']
              + ['Conditions/Alignment/Rich2/HPD%d_Align' %i for i in range(196, 484)]
              + ['Conditions/Alignment/Rich2/HPDPanel%d_Align' %i for i in range(0, 2)]
    }  
  #print "INFO: (RichAnalyzer.py) hpdConds = ", hpdConds

  ## Create a Dictionary that combines mirrAlignConds and hpdConds
  allFileConds = {}
  for x,y in mirrAlignConds.items():
    if x in hpdConds.keys():
      allFileConds[x] = mirrAlignConds[x] + hpdConds[x]
    else:
      allFileConds[x] = mirrAlignConds[x]
  for x,y in hpdConds.items():
    if x not in mirrAlignConds.keys():
      allFileConds[x] = hpdConds[x]
  #print "INFO: (RichAnalyzer.py) allFileConds = ", allFileConds

  # Get the work directory ususally /group/online/AligWork/Rich1/ or /group/online/AligWork/Rich2/
  if hasattr(Online, "AlignXmlDir"):
    xmlDir = Online.AlignXmlDir
  else:
    xmlDir = workdir
  print "INFO: (RichAnalyzer.py) xmlDir = ", xmlDir

  pat = os.path.join(xmlDir, "CondDB_%s.xml")
  print "INFO: (RichAnalyzer.py) pat = ", pat
  
  # baseloc = '/group/online/hlt/conditions'  
  baseloc = CondDB().getProp( "DBSnapshotDirectory" ) 
  print "INFO: (RichAnalyzer.py) baseloc = ", baseloc

  ## Get the dictionary of conditions for the RunChangeHandler service
  ## from the file used by HLT2. Messing with sys.path is evil, but
  ## perhaps the least evil here...
  import sys
  try:
    import All
  except ImportError:
    rd = '/group/online/hlt/conditions/RunChangeHandler'
    sys.path.append(rd)
    import All
    print "INFO: (RichAnalyzer.py) Had to use sys.path.append(", rd, ") to make import All work."

  ### Add base path of the files to the files in All.py
  conditionmap = {os.path.join(baseloc, f) : cs for f, cs in All.ConditionMap.iteritems()}
  #print "INFO: (RichAnalyzer.py) conditionmap = ", conditionmap
  
  ## Pre-gitDB: Uncomment this if running on 2015 data instead of 2016 data
  #conditionmap = {k.replace('2016', '2015') : v for k, v in conditionmap.iteritems()}

  ## Create a specific conditions map with only mirrAlignConds for the Rich detector we are using
  mirrAlignConditionmap = {pat % sd : f for (sd, f) in mirrAlignConds.iteritems() if sd == ('Rich' + str(whichRich))}
  #print "INFO: (RichAnalyzer.py) mirrAlignConditionmap = ", mirrAlignConditionmap

  # Configure RunChangeHandlerSvc by hand to allow different locations of the
  # files. postConfigAction hammer required...
  def applyConds():
    from Configurables import RunChangeHandlerSvc
    rch = RunChangeHandlerSvc()
    Gaudi.ApplicationMgr().ExtSvc.append(rch)
    rch.Conditions = {c : f for f, cs in conditionmap.iteritems() for c in cs}
    #print "INFO: (RichAnalyzer.py) rch.Conditions without all conditions in allFileConds removed and without our conditions added = ", rch.Conditions

    # pop all allFileConds conditions out of the rch.Conditions dictionary (we don't want Brunel to access rich1mirroralign.xml or rich2mirroralign.xml in /group/online/hlt/conditions/LHCb/NoYear/first_three_digits_of_run/run/, where run is the run number)                                                                                     
    for c,f in (rch.Conditions.copy()).iteritems():
      for l in allFileConds.values():
        if c in l:
          rch.Conditions.pop(c, None)
    #print "INFO: (RichAnalyzer.py) rch.Conditions without our conditions added = ", rch.Conditions

    # add *only* the main mirror alignment conditions for the Rich that we are aligning
    rch.Conditions.update({c : f for f, cs in mirrAlignConditionmap.iteritems() for c in cs})
    #print "INFO: (RichAnalyzer.py) rch.Conditions = ", rch.Conditions

  #Send rch.Conditions to the RunChangeHandler
  appendPostConfigAction(applyConds)

  # Turn off sequences we do not need

  def turnOffL0Decode():
    from Configurables import GaudiSequencer
    GaudiSequencer("DecodeTriggerSeq").Members = []
  appendPostConfigAction(turnOffL0Decode)

  def turnOffLumiSeq() :
    from Configurables import GaudiSequencer
    GaudiSequencer("LumiSeq").Members = []
  appendPostConfigAction(turnOffLumiSeq)

  def turnOffNotPhysicsSeq() :
    from Configurables import GaudiSequencer
    GaudiSequencer("NotPhysicsSeq").Members = []
  appendPostConfigAction(turnOffNotPhysicsSeq)

  # Selections

  from Configurables import MagneticFieldSvc
  MagneticFieldSvc().UseSetCurrent = True

  brunel.WriteFSR  = False # This crashes Jaap's stuff
  EventLoopMgr().OutputLevel = MSG_WARNING
  EventLoopMgr().Warnings    = False

  # Hack by Chris
  from Configurables import RichRecSysConf
  rConf = RichRecSysConf("RichOfflineRec")
  rConf.setProp("PidConfig", "None")

  # Need a postConfigAction to apply the correct condtions
  minP = 20 # default: RICH1
  if whichRich == 2:
    minP = 40
  appendPostConfigAction(ChangeP(minP * GeV))

  from Configurables import  TrackSys
  TrackSys().setProp( "TrackPatRecAlgorithms", ["FastVelo","VeloTT","ForwardHLT1","ForwardHLT2"] )

  rConf.photonConfig().SelectionMode = "Wide"

  if whichRich == 2:
      rConf.Radiators = [ "Rich2Gas" ]
  if whichRich == 1:
      rConf.Radiators = [ "Rich1Gas" ]

  rConf.trackConfig().TrackCuts = { "Forward" : { "Chi2Cut" : [0,5], "PCut" : [minP,9999999] },
                                    "Match"   : { "Chi2Cut" : [0,5], "PCut" : [minP,9999999] } }
  rConf.TracklessRingAlgs = [ ]

  from Configurables import HistogramPersistencySvc
  HistogramPersistencySvc().OutputFile = ""

  from Configurables import RichRecQCConf
  RichRecQCConf ("OfflineRichMoni").removeMonitor("L1SizeMonitoring"       )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("DBConsistencyCheck"     )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("HotPixelFinder"         )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("PidMonitoring"          )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("PixelMonitoring"        )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("TrackMonitoring"        )
  ## Commented the next line out on 2016-04-08, in order to create PhotonMonitoring plots. But uncommented 2017-03-22 as we now have our own such monitor #####  
  RichRecQCConf ("OfflineRichMoni").removeMonitor("PhotonMonitoring"       )
  ## 2017-03-22 We now also want our special monitor with RichRecQC "TightMirror" cuts [similar to hacked RichRecQC we used in 2016] to run. #####  
  RichRecQCConf ("OfflineRichMoni").addMonitor("PhotonMonitoringAlignOnline") # i.e. this is NOT a default monitor
  ## Added the next line on 2016-04-08, to Open up the CK res plot range, for the Wide photon selection
  RichRecQCConf ("OfflineRichMoni").CKThetaResRange = ( 0.05, 0.008, 0.004 )
  ##############################################################################################
  RichRecQCConf ("OfflineRichMoni").removeMonitor("TracklessRingAngles"      )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("TracklessRingPeakSearch"  )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("HPDImageShifts"           )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("DataDecodingErrors"       )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("HPDIFBMonitoring"         )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichPixelPositions"       )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("HPDHitPlots"              )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichTrackGeometry"        )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichGhostTracks"          )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichCKThetaResolution"    )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichTrackResolution"      )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichPhotonSignal"         )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichTrackCKResolutions"   )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichPhotonGeometry"       )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("PhotonRecoEfficiency"     )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichPhotonTrajectory"     )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichStereoFitterTests"    )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichPhotonRayTracingTests")
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichPhotonRecoCompare"    )
  # These below are gone now, but no harm in leaving these lines here
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichRayTracingTests"      )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichDataObjectChecks"     )
  RichRecQCConf ("OfflineRichMoni").removeMonitor("RichRecoTiming"           )

  from Configurables import RichAlignmentConf
  RichAlign = RichAlignmentConf ("OfflineRichMoni_RichAlignmentConf")
  if whichRich == 2:
      RichAlign.Radiators =  ["Rich2Gas"]
  if whichRich == 1:
      RichAlign.Radiators =  ["Rich1Gas"]
  RichAlign.HPDList =  [[0],[0],[0]]
  RichAlign.Histograms = "Expert"

  RichAlign.NTupleProduce = False
  RichAlign.R1NTupleProduce = False

  deltatheta = alignConf.getProp("deltaThetaWindow")/1000.
  RichAlign.DeltaThetaRange = [ 0.04, deltatheta, deltatheta ]

  brunel.OutputLevel       = MSG_INFO
  brunel.PrintFreq         = 100

  HistogramPersistencySvc().OutputLevel = MSG_ERROR
  from Configurables import CondDB
  from Configurables import CondDBAccessSvc
  from Configurables import ApplicationMgr
  from Configurables import HistogramPersistencySvc

  import uuid
  runType="Rich"+str(whichRich) + "_"+str(uuid.uuid4())+"_"+str(n)
  histoDir = workdir
  appendPostConfigAction(saveHistograms(runType, histoDir, n))

  print 'INFO: brunel = ', brunel
  return brunel

def setupOnline(directory, filename, prefix, alignConf):
  import OnlineEnv as Online
  from Configurables import LHCb__FILEEvtSelector as es
  app=Gaudi.ApplicationMgr()
  app.AppName = ''
  app.HistogramPersistency = 'NONE'
  app.SvcOptMapping.append('LHCb::FILEEvtSelector/EventSelector')
  app.SvcOptMapping.append('LHCb::FmcMessageSvc/MessageSvc')
  from Configurables import MirrPauseHandler as mp
  mp().ReferenceRunNr = Online.DeferredRuns[0] if hasattr(Online, 'DeferredRuns') else 0
  mp().ItNrFile = alignConf.getProp('ItNrFile')
  app.TopAlg.append("MirrPauseHandler")
  sel = es("EventSelector")
  sel.AllowedRuns = Online.DeferredRuns if hasattr(Online, 'DeferredRuns') else []
  sel.Directory = directory
  sel.FilePrefix = prefix
  sel.Input = filename
  sel.DeleteFiles = False;
  sel.Decode = False
  sel.Pause = True
  sel.PauseSleep = 5000
  app.EvtSel  = sel
  app.EvtMax = alignConf.getProp('EvtMax')
  Online.rawPersistencySvc()
  evtloop = Configs.EventLoopMgr('EventLoopMgr')
  evtloop.Warnings = False
  evtloop.EvtSel = sel
  app.EventLoop = evtloop
  runable = Configs.LHCb__EventRunable("Runable")
  runable.MEPManager        = ""
  app.Runable = runable
  app.AuditAlgorithms = False
  Configs.MonitorSvc().OutputLevel = MSG_ERROR
  Configs.MonitorSvc().UniqueServiceNames = 1
  Configs.MonitorSvc().DimUpdateInterval = -1
  Configs.RootHistCnv__PersSvc("RootHistSvc").OutputLevel = MSG_ERROR
  app.OutputLevel = MSG_INFO
  print 'INFO: sel = ', sel

#============================================================================================================
def patchMessages():
  import OnlineEnv as Online
  app=Gaudi.ApplicationMgr()
  Configs.AuditorSvc().Auditors = []
  app.MessageSvcType = 'LHCb::FmcMessageSvc'
  if Gaudi.allConfigurables.has_key('MessageSvc'):
    del Gaudi.allConfigurables['MessageSvc']
  msg = Configs.LHCb__FmcMessageSvc('MessageSvc')
  msg.fifoPath      = os.environ['LOGFIFO']
  msg.LoggerOnly    = True
  msg.doPrintAlways = False
  msg.OutputLevel   = MSG_INFO # Online.OutputLevel

#============================================================================================================
def start():
  Online = importOnline()
  import sys
  Online.end_config(False)

#============================================================================================================
def doIt(filename = "", n = 0, whichRich = 1):

  if whichRich == 1:
    from Configurables import Rich1MirrAlignOnConf
    alignConf = Rich1MirrAlignOnConf()
  elif whichRich == 2:
    from Configurables import Rich2MirrAlignOnConf
    alignConf = Rich2MirrAlignOnConf()
  alignConf.__apply_configuration__()

  workdir = alignConf.getProp('WorkDir')
  true_online = os.environ.has_key('LOGFIFO') and (os.environ.has_key('PARTITION') or os.environ.has_key('PARTITION_NAME'))

  if os.path.isdir(filename):
    directory = filename
    prefix = 'Run_'
  else:
    directory = os.path.dirname(filename)
    prefix = os.path.basename(filename)[:4]

  br = patchBrunel(true_online, alignConf, n)
  setupOnline(directory, filename, prefix, alignConf)
  if true_online:
    patchMessages()
  start()
