import os
import shutil
import re
import time
from   time       import time
from   time       import gmtime, strftime
import subprocess, shlex
import sys
from ROOT import TH1, TFile

class SetupHelper:
    def __init__(self, _alignConf):
        self.alignConf = _alignConf
        # maybe put combAndMirrSubsets here later somehow
        self.nameStr = self.alignConf.getProp('nameStr')
        self.tiltNames = self.alignConf.getProp('tiltNames')
        self.workdir = self.alignConf.getProp('WorkDir')
        self.whichRich = self.alignConf.getProp('Rich')
        self.autoUpdate = self.alignConf.getProp('autoUpdate')
        self.magnFactorsMode = self.alignConf.getProp('magnFactorsMode')
        self.minAverageBinPop = self.alignConf.getProp('minAverageBinPop')
        self.phiBinFactor = self.alignConf.getProp('phiBinFactor')
        self.deltaThetaWindow = self.alignConf.getProp('deltaThetaWindow')
        self.combinFitMethod = self.alignConf.getProp('combinFitMethod')
        self.solutionMethod = self.alignConf.getProp('solutionMethod')
        self.coeffCalibTilt = self.alignConf.getProp('coeffCalibTilt')
        self.magnifDir = self.alignConf.getProp('magnifDir') 
        self.fixSinusoidShift = self.alignConf.getProp('fixSinusoidShift')
        self.fileprename = self.workdir + '/' + "RichRecQCHistos_rich" + str(self.whichRich) + "_" + self.nameStr + "_" +'i' 
        self.experts = ['Paras.Naik@cern.ch','samuel.maddrell-mander@cern.ch','matthew.george.chapman@cern.ch','claire.prouve@cern.ch']
        self.developers = ['lhcb-rich-mirror-alignment-development@cern.ch']
        self.emails = ['lhcb-rich-mirror-alignment-development@cern.ch','jonesc@hep.phy.cam.ac.uk','antonis.papanestis@stfc.ac.uk','m.mccann@imperial.ac.uk','silvia.gambetta@cern.ch']
        self.directoryTime = strftime("%Y%m%d_%H%M%S", gmtime())
        self.savedir = self.alignConf.getProp('SaveDir') + 'Rich' + str(self.whichRich) + '/' + self.directoryTime + '/'

    def setupMagnifFiles(self):
        magnifDir = self.alignConf.getProp('magnifDir')
        for f in os.listdir(magnifDir):
            shutil.copy(magnifDir + "/" + f, self.workdir)
    
    def startMinIt(self, m_it):
        removeIt = (m_it - m_it % 9) / 9
        files = os.listdir(self.workdir)
        
        re_fitFolders = re.compile(r"Rich" + str(self.whichRich) + "MirrCombinFit_" + self.nameStr + "[\w\.]*_i" + str(removeIt))
        re_alignFiles = re.compile(r"Rich" + str(self.whichRich) + "MirrAlign[a-zA-z]*_i" + str(removeIt) + "\.txt")
        fitFolders = [x for x in files if re_fitFolders.match(x) ]
        alignFiles = [x for x in files if re_alignFiles.match(x) ]
        
        if (m_it % 9 == 0):
            connectStr = ""
        else:
            connectStr = "_"
        rootFile = "RichRecQCHistos_rich" + str(self.whichRich) + "_" + self.nameStr + connectStr + self.tiltNames[m_it % 9] + "_i" + str(removeIt) + ".root"
        
        for f in fitFolders:
            shutil.rmtree(self.workdir + "/" + f)
        for f in alignFiles:
            os.remove(self.workdir + "/" + f)
        if os.path.exists(self.workdir + "/" + rootFile):
            os.remove(self.workdir + "/" + rootFile)

    def finalize(self, conv, n_it, compareXML, farm_config_time, elapsed_runtimes, DBUpdated, nEventsListList):
        nEventsList = nEventsListList[0]
        nEvents = nEventsListList[1]

        if not os.path.exists(self.savedir):
            os.makedirs(self.savedir)
        for f in os.listdir(self.workdir):
            if os.path.isdir(self.workdir + "/" + f):
                shutil.copytree(self.workdir + "/" + f, self.savedir + '/' + f)
            else:
                shutil.copy2(self.workdir + "/" + f, self.savedir)
        print "INFO: savedir " + self.savedir

        summaryFile = self.workdir + "/summary.txt"
        self.writeSummary(conv, n_it, farm_config_time, elapsed_runtimes, summaryFile, DBUpdated, nEvents, nEventsList)

        shutil.copy2(summaryFile, self.savedir)

        from AlignSummary import AlignSummary
        alignSummary = AlignSummary(self.alignConf, n_it)
        alignSummary.makeSummaryPlot(compareXML)

        ChangeWRTDBFile  = self.workdir + "/Rich" + str(self.whichRich) + '_ChangeWRTDB_'+ str(n_it + 1) +'.txt'
        shutil.copy2(ChangeWRTDBFile, self.savedir)
        AlignSummaryFile = self.workdir + "/Rich" + str(self.whichRich) + '_AlignSummary.pdf'    
        shutil.copy2(AlignSummaryFile, self.savedir)

        # Write summary.txt, _AlignSummary.pdf, and _ChangeWRTDB.txt to the ELOG
        alignment_time = elapsed_runtimes['elapsed_time_run']
        self.writeInLogbook(conv, n_it, farm_config_time, alignment_time, summaryFile, ChangeWRTDBFile, AlignSummaryFile, DBUpdated, nEvents)

    def writeInLogbook(self, conv, n_it, farm_config_time, alignment_time, summaryFile, ChangeWRTDBFile, AlignSummaryFile, DBUpdated, nEvents):
        hh, r = divmod(farm_config_time, 3600)
        mm, ss = divmod(r, 60)
        farm_config_time_str = "{:0>2}:{:0>2}:{:05.2f}".format(int(hh),int(mm),ss)
        hh, r = divmod(alignment_time, 3600)
        mm, ss = divmod(r, 60)
        alignment_time_str = "{:0>2}:{:0>2}:{:05.2f}".format(int(hh),int(mm),ss)
        host = 'lblogbook.cern.ch' 
        port = 8080
        username = 'common Common\\!'
        logbook = 'RICH'
        author = 'MirrAlign Monitor' # Or any other name
        activity = 'Alignment'
        subject = ''
        text = ''
        instruction_file = 'https://twiki.cern.ch/twiki/bin/view/LHCb/LHCbRichMirrorAlignShiftInfo'
        subjText = ''
        if DBUpdated:
            subjText += ' updated,'
        if DBUpdated is None:
            subjText += ' needs MirrAlign expert decision,'
        else:
            subjText += ''
        if self.alignConf.getProp('testing') is True:
            subject += 'TEST [' + self.alignConf.getProp('dataVariant') + ']: '
        text += "The Mirror "
        text += str(activity)
        text += " for RICH"
        text += str(self.whichRich)
        if conv:
            text += " ran for "
            text += alignment_time_str
            text += " (excluding an LHCbA configuration time of "
            text += farm_config_time_str
            text += ") and converged. "
            subject += 'RICH' + str(self.whichRich) + ' alignment ' + self.directoryTime + subjText + ' converged'
        else:
            text += " ran for "
            text += alignment_time_str
            text += " (excluding an LHCbA configuration time of "
            text += farm_config_time_str
            text += ") and did not converge. "
            subject += 'RICH' + str(self.whichRich) + ' alignment ' + self.directoryTime + subjText + ' incomplete'
        text += "We processed approximately "
        text += str(nEvents)
        text += " events per iteration from the RICH"
        text += str(self.whichRich)
        text += "Decision of our Hlt1CalibRICHMirror HLT line. "
        text += str(n_it + 1)
        text += " iteration(s) occured. Mirror "
        text += str(activity)
        text += " monitoring information and plots are attached; shifter's instructions can be found at "
        text += str(instruction_file)
        text += " and the output directory timestamp is "
        text += self.directoryTime
        text += "."
        if DBUpdated:
            text += " Due to a significant change from the alignment in the DB, or a change in magnet polarity, the DB was automatically updated." # We should probably provide more info here
        if DBUpdated is None:
            text += " There was a significant change from the alignment in the DB, or a change in magnet polarity. However the automatic update feature is off (or was disabled during the alignment), so the DB was not updated. Please notify a MirrAlign expert to investigate whether this alignment should be installed." # We should probably provide more info here
        command = 'elog -h "'
        command += str(host)
        command += '" -p '
        command += str(port)
        command += ' -l "'
        command += str(logbook)
        command += '" -u '
        command += str(username)
        command += ' -a "Author='
        command += str(author)
        command += '" -a "System='
        command += str(activity)
        command += '" -a "Subject='
        command += str(subject)
        command += '" "'
        command += str(text)
        command += '"'
        if os.path.exists(summaryFile):
            command += ' -f "{0}"'.format(summaryFile)
        if os.path.exists(ChangeWRTDBFile):
            command += ' -f "{0}"'.format(ChangeWRTDBFile)
        if os.path.exists(AlignSummaryFile):
            command += ' -f "{0}"'.format(AlignSummaryFile)
        cmd = shlex.split(command)
        FNULL = open(os.devnull, 'w')
        subprocess.call(cmd, stdout=FNULL, stderr=subprocess.STDOUT)

        subj2 = ''
        if self.alignConf.getProp('testing') is True:
            subj2 += 'TEST [' + self.alignConf.getProp('dataVariant') + ']: '        
        text2 = "The Mirror "
        text2 += str(activity)
        text2 += " for RICH"
        text2 += str(self.whichRich)
        if conv:
            text2 += " ran for "
            text2 += alignment_time_str
            text2 += " and converged. "
            subj2 += 'RICH{0} alignment {1} converged: New ELOG entry'.format(self.whichRich,self.directoryTime)
        else:
            text2 += " ran for "
            text2 += alignment_time_str
            text2 += " and did not converge. Approximately "
            subj2 += 'RICH{0} alignment {1} incomplete: New ELOG entry'.format(self.whichRich,self.directoryTime)
        text2 += str(nEvents)
        text2 += " events per iteration were processed. "
        text2 += str(n_it + 1)
        text2 += " iteration(s) occured. \n"
        if DBUpdated:
            text2 += "\n Due to a significant change from the alignment in the DB, or a change in magnet polarity, the DB was automatically updated. \n" # We should probably provide more info here
        if DBUpdated is None:
            text2 += "\n There was a significant change from the alignment in the DB, or a change in magnet polarity. However the automatic update feature is off (or was disabled during the alignment), so the DB was not updated. Please notify a MirrAlign expert to investigate whether this alignment should be installed. \n" # We should probably provide more info here
        text2 += "\n More information is available at the following link: \n\n "
        text2 += 'https://lblogbook.cern.ch/RICH/?System=%5EAlignment%24&Subject={0}&mode=full'.format(self.directoryTime)
        
        self.sendEmail(subj2, text2, self.emails)
  
        return 0

    def writeSummary(self, conv, n_it, farm_config_time, elapsed_runtimes, summaryFile, DBUpdated, nEvents, nEventsList):
        hh, r = divmod(farm_config_time, 3600)
        mm, ss = divmod(r, 60)
        farm_config_time_str = "{:0>2}:{:0>2}:{:05.2f}".format(int(hh),int(mm),ss)
        hh, r = divmod(elapsed_runtimes['elapsed_time_run'], 3600)
        mm, ss = divmod(r, 60)
        alignment_time_str = "{:0>2}:{:0>2}:{:05.2f}".format(int(hh),int(mm),ss)
        MirrCombinFit_time_str = str(elapsed_runtimes['elapsed_times_RichMirrCombinFit'])
        MirrAlign_time_str = str(elapsed_runtimes['elapsed_times_RichMirrAlign'])
        
        import OnlineEnv as Online
        runs = Online.DeferredRuns
        runsString = ''
        fillsString = ''
        polaritiesString = ''
        for i in range(0, len(runs)):
            runsString += str(runs[i]) + '  ' 
            fill = self.getFillNumber(runs[i])
            if fill is not None:
                fillsString += '  ' + str(fill) + '  '
            else:
                fillsString += '  ????  '
            polarity = self.getPolarity(runs[i])
            if polarity == 'UP':
                polaritiesString += '    UP  '
            elif polarity == 'DOWN':
                polaritiesString += '  DOWN  '
            elif polarity == 'OFF': 
                polaritiesString += '   OFF  '
            else: #something went wrong
                polaritiesString += '  ????  '
            
        if os.path.exists(summaryFile) :
            os.remove(summaryFile)
        f = open(summaryFile, 'w')
        args  = ''
        args += 'richDetector            = ' + str(self.whichRich) + '\n'
        args += 'DB updated              = ' + str(DBUpdated) + '\n'
        if DBUpdated is None:
            args += '                          (Alignment changed, but autoUpdate is False or was disabled) \n '
        args += 'converged               = ' + str(conv) + '\n'
        args += 'Number of iterations    = ' + str(n_it + 1) + '\n'
        args += 'Number of events        = ' + str(nEvents) + '\n'
        args += 'runs                    = ' + runsString + '\n'
        args += 'fills                   = ' + fillsString + '\n'
        args += 'polarities              = ' + polaritiesString + '\n'
        args += 'dataVariant             = ' + str(self.alignConf.getProp('dataVariant')) + '\n'
        args += 'magnFactorsMode         = ' + str(self.magnFactorsMode) + '\n'
        args += 'savedir                 = ' + self.savedir + '\n'
        print "INFO: LHCbA config time (readable): " + farm_config_time_str
        # print "INFO: Note, the LHCbA config time may be inaccurate if the alignment is run manually with LHCbA AutoPilot OFF."
        args += 'LHCbA config time       = ' + farm_config_time_str + '\n'
        print "INFO: Alignment time    (readable): " + alignment_time_str
        args += 'Alignment time          = ' + alignment_time_str + '\n'
        args += '\n'
        args += 'Final iteration label   = ' + 'i' + str(n_it) + '\n'
        args += 'autoUpdate              = ' + str(self.autoUpdate) + '\n'
        args += 'testing                 = ' + str(self.alignConf.getProp('testing')) + '\n'
        args += 'requiredEvents          = ' + str(self.alignConf.getProp('requiredEvents')) + '\n'
        args += 'magnifDir               = ' + str(self.magnifDir) + '\n'
        args += 'minAverageBinPop        = ' + str(self.minAverageBinPop) + '\n'
        args += 'phiBinFactor            = ' + str(self.phiBinFactor) + '\n'
        args += 'deltaThetaWindow        = ' + str(self.deltaThetaWindow) + '\n'
        args += 'combinFitMethod         = ' + str(self.combinFitMethod) + '\n'
        args += 'solutionMethod          = ' + str(self.solutionMethod) + '\n'
        args += 'coeffCalibTilt          = ' + str(self.coeffCalibTilt) + '\n'
        args += 'fixSinusoidShift        = ' + str(self.fixSinusoidShift) + '\n'
        args += 'maxIterations           = ' + str(self.alignConf.getProp('maxIterations')) + '\n'
        args += 'EvtMax                  = ' + str(self.alignConf.getProp('EvtMax')) + '\n'        
        args += 'regularizationMode      = ' + str(self.alignConf.getProp('regularizationMode')) + '\n'
        args += 'stopToleranceMode       = ' + str(self.alignConf.getProp('stopToleranceMode')) + '\n'
        args += 'nominalResolutionSigma  = ' + str(self.alignConf.getProp('nominalResolutionSigma')) + '\n'
        args += 'stopSigmaFraction       = ' + str(self.alignConf.getProp('stopSigmaFraction')) + '\n'
        args += 'stopTolerancePriY       = ' + str(self.alignConf.getProp('stopTolerancePriY')) + '\n'
        args += 'stopTolerancePriZ       = ' + str(self.alignConf.getProp('stopTolerancePriZ')) + '\n'
        args += 'stopToleranceSecY       = ' + str(self.alignConf.getProp('stopToleranceSecY')) + '\n'
        args += 'stopToleranceSecZ       = ' + str(self.alignConf.getProp('stopToleranceSecZ')) + '\n'
        args += 'nEventsList             = ' + str(nEventsList) + '\n'
        args += 'CombinFit times         = ' + MirrCombinFit_time_str + '\n'
        args += 'MirrAlign times         = ' + MirrAlign_time_str + '\n'
        f.write(args)
        f.close()
        sys.stdout.flush()
        
    def sendEmail(self, subject, text, receiversList): 
        import smtplib
        sender = 'lhcb-rich-mirror-alignment-development@cern.ch'
        message = 'From: MirrAlign Monitor <'
        message += sender
        message += '>\n'
        message += 'To:'
        for ie in receiversList:
            message += ' <'
            message += ie
            message += '>,'
        message = message[:-1]
        message += '\n'
        message += 'Subject: ' 
        message += subject 
        message += '\n\n'
        message += '\n{0}\n\n'.format(text)
        try:
            smtpObj = smtplib.SMTP('localhost')
            smtpObj.sendmail(sender, receiversList, message)         
            print "INFO: Successfully sent email"
        except smtplib.SMTPException:
            print "ERROR: Unable to send email"

    # Engineered from https://gitlab.cern.ch/lhcb/Lovell/blob/master/VeloAnalysisFramework/python/veloview/utils/rundb.py            
    def getFillNumber(self, runNumber):
        response = self.queryRunDB(runNumber)
        # Check for null responses
        if not response:
            return None
        try:
            fill = response['fillid']
        except KeyError:
            # Fill ID not included in JSON response
            return None
        return fill

    # Engineered from https://gitlab.cern.ch/lhcb/Lovell/blob/master/VeloAnalysisFramework/python/veloview/utils/rundb.py
    def getPolarity(self, runNumber):
        # only possible polarity states: UP, DOWN, OFF
        response = self.queryRunDB(runNumber)
        # Check for null responses
        if not response:
            return None
        try:
            polarity = response['magnetState']
        except KeyError:
            # Polarity not included in JSON response
            return None
        # API returns UP for magnet up, DOWN for magnet down, OFF for off
        return polarity

    # Engineered from https://gitlab.cern.ch/lhcb/Lovell/blob/master/VeloAnalysisFramework/python/veloview/utils/rundb.py
    def queryRunDB(self, runNumber):
        import json, urllib
        RUNDB_API_URL = 'http://lbrundb.cern.ch/api/run/{0}/' # e.g. see http://lbrundb.cern.ch/api/run/185380/ for what's available
        request = urllib.urlopen(RUNDB_API_URL.format(runNumber))
        # Make sure the request was OK, not e.g. 404 not found
        if request.getcode() != 200:
            return None
        try:
            run_json = json.load(request)
        except ValueError:
            # Couldn't decode the response
            return None
        return run_json
    
    def getNEventsListList(self, n_it):
        nEventsList = []
        filename = []
        thisFile = []
        hist = []
        for j in range(0, n_it + 1):
            filename.append( self.fileprename + str(j) + '.root' )
            thisFile.append( TFile(filename[j], 'read') )
            hist.append( thisFile[j].Get('RICH/RichODIN/TriggerType') )
            nEventsList.append(int(hist[j].GetEntries()))
        nEvents = int( round( float(sum(nEventsList)) / float(n_it + 1) ) ) # If mag factors float, this only reports for the data with untilted XML
        return [nEventsList , nEvents]
