#Example run options for RichTimingScanAnalysis code
#Conor Fitzpatrick
from Panoptes.Configuration import *
from Configurables import RichTimingScanAnalysis
from Configurables import LHCbApp
from glob import glob
import os

#see if values have been set on the environment, if not use default values
nSteps         = os.environ.get('TIMING_SCAN_STEPS',           51)     # Number of steps taken
nEventsPerStep = os.environ.get('TIMING_SCAN_EVENTS_PER_STEP', 20000)  # Number of events/per step
stepWidth      = os.environ.get('TIMING_SCAN_STEP_WIDTH',      1)      # Step width (in ns for fine time, or in 25ns steps = 1 coarse time step)
stepStart      = os.environ.get('TIMING_SCAN_FIRST_STEP',     -25)     # Starting point in coarse time steps, or ns. 
run            = os.environ.get('TIMING_SCAN_RUN',             183911) # run number

from Configurables import CondDB, CondDBAccessSvc
Panoptes().DDDBtag   = "dddb-20150724"
Panoptes().CondDBtag = "cond-20161011"
#CondDB().Online = True

#customDBs = glob('/group/rich/ActiveDBSlices/*.db')
#for db in customDBs:
#  CondDB().addLayer( CondDBAccessSvc(os.path.basename(db), ConnectionString="sqlite_file:"+db+"/LHCBCOND", DefaultTAG="HEAD") )
Panoptes().UseOracle=False
CondDB().IgnoreHeartBeat=True
CondDB().EnableRunStampCheck = False

#Fill timing histos for Rich 1
timescanR1 = RichTimingScanAnalysis("RichTimingScanAnalysis_Rich1")
timescanR1.NEventsPerStep = nEventsPerStep
timescanR1.NSteps = nSteps
timescanR1.WhichRich = 1
timescanR1.StepWidth = stepWidth
timescanR1.StepStart = stepStart
#timescanR1.OutputLevel = DEBUG

#Fill timing histos for Rich 2
timescanR2 = RichTimingScanAnalysis("RichTimingScanAnalysis_Rich2")
timescanR2.NEventsPerStep = nEventsPerStep
timescanR2.NSteps = nSteps
timescanR2.WhichRich = 2
timescanR2.StepWidth = stepWidth
timescanR2.StepStart = stepStart
#timescanR2.OutputLevel = DEBUG

RichMonitoringSysConf().MonitorReconstruction = False
RichMonitoringSysConf().MonitorImageMovements = False
RichMonitoringSysConf().MonitorRaw            = False
RichMonitoringSysConf().MonitorDAQ            = False 
RichMonitoringSysConf().MonitorHitMaps        = False
RichMonitoringSysConf().MonitorNhits          = False
RichMonitoringSysConf().MonitorCalibration    = False

# Input datafile to process
selectorstr = " SVC='LHCb::MDFSelector'"
inputfiles = []
files = glob('/daqarea/lhcb/data/*/RAW/FULL/RICH/TEST/'+str(run)+'/'+str(run)+'_*.raw')
for f in files:
  inputfiles.append("DATAFILE='PFN:" + f + "'" + selectorstr)
print inputfiles
EventSelector().Input = inputfiles

# Should be -1 or the total number of steps * evts/step
Panoptes().EvtMax = -1
LHCbApp().DataType = "2016"
EventSelector().PrintFreq = 5000

def dotimescan(): 
    ApplicationMgr().TopAlg += [ timescanR1, timescanR2 ] #Both RICHes analysed
#    ApplicationMgr().TopAlg += [ timescanR1 ] #Just RICH1
#    ApplicationMgr().TopAlg += [ timescanR2 ] #Just RICH2

appendPostConfigAction(dotimescan)


