//TimeScan.cc: Part of the RichTimingScanAnalysis package
// Conor Fitzpatrick, 10/03/2010
//conor.fitzpatrick@cern.ch

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <iomanip>
#include "TFile.h"
#include "TTree.h"
#include "TROOT.h"
#include "TSystem.h"
#include "TStyle.h"
//#include "TCint.h"
#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TAxis.h"
#include <TRandom3.h>
#include <TArrow.h>
#include <TKey.h>
#include <TPaveStats.h>
#include <TLegend.h>
#include <vector>
#include "TMath.h"
//using namespace std;
using std::cout;
using std::endl;
using std::vector;

typedef vector<Double_t> array1d;
typedef vector<TString> array1s;

int main(int argc, char *argv[]){
	if(argc !=5 && argc !=7){
		cout << argv[0] << ": Part of the RichTimingScanAnalysis package" << endl;
		cout << "Plots the per-RICH HPD + L0 board turn-on profiles, and writes to stdout the current timing information" << endl;
		cout << "Usage: " << argv[0] << " <inputHistosFromPanoptes.root> <Threshold (usually 0.9)> <outputFilesName> <CoarseTime/FineTime1> [minstep] [maxstep]" << endl; 
		return EXIT_FAILURE;
	}
	TString scanType = argv[4];
	if(scanType != "CoarseTime" && scanType != "FineTime1"){
		cout << argv[0] << ": Part of the RichTimingScanAnalysis package" << endl;
		cout << "Plots the per-RICH HPD + L0 board turn-on profiles, and writes to stdout the current timing information" << endl;
		cout << "Usage: " << argv[0] << " <inputHistosFromPanoptes.root> <Threshold (usually 0.9)> <outputFilesName> <CoarseTime/FineTime1> [minstep] [maxstep]" << endl;
		return EXIT_FAILURE;
	}


	Int_t minbin = -25;
	Int_t maxbin = 25;
	if(argc==7){
		minbin = atoi(argv[5]);
		maxbin = atoi(argv[6]);
	}
	gROOT->SetStyle("Plain");

	gStyle->SetFrameFillColor(0);
	gStyle->SetPadColor(0);
	gStyle->SetFillColor(10);

	Double_t onfrac = atof(argv[2]); // What % do we consider the HPD/board turned on at?

	TString outputname = argv[3];
	TFile *output = new TFile(outputname + ".root","RECREATE");
	TFile *input = new TFile(argv[1],"READ");

	//Loop over riches
	for(UInt_t Rich = 1; Rich<3; Rich++){
		TString location = "RICH/RichTimingScanAnalysis_Rich";
		location += Rich;

		TString DBFileName = outputname; 
		DBFileName += "_RICH";
		DBFileName += Rich;
		if(input->cd(location)){
			cout << "Processing Rich " << Rich << endl;
			TString RichName = "RICH";
			RichName += Rich;
			// Get all keys in current dir
			TDirectory *current_sourcedir = gDirectory;
			TIter nextkey( current_sourcedir->GetListOfKeys() );
			TKey *key;
			output->cd();
			TDirectory *outputdir = output->mkdir(RichName,RichName);
			outputdir->cd();

			Double_t min, max, delta, onval, midpoint;
			// Store midpoints, delays, IDs of L0 boards and HPDs
			array1d L0midpoints, L0delays;
			array1s L0IDs;
			array1d HPDmidpoints, HPDdelays;
			array1s HPDIDs;

			// Average midpoints
			Double_t HPDavemidpoint = 0;
			Double_t L0avemidpoint = 0;

			// Bin in which HPD turns on/off
			Int_t turnon = 0;
                        Int_t turnoff = 0;
			TH1I *hpdoff = 0;
                        TH1I *l0off = 0;
                        TH1I *hpdon = 0;
                        TH1I *l0on = 0;
			TH1F *hpdmid = 0; 
                        TH1F *l0mid = 0; 
			TH1F *globalHits = 0;
			Double_t occupancy = 0;
			Double_t doccupancy = 0;
			Bool_t madeAxes = false;
			Double_t hpd1ony = 0;
                        Double_t hpd1offy = 0;
                        Double_t hpd1midy = 0;
                        Double_t hpd2ony = 0;
                        Double_t hpd2offy = 0;
                        Double_t hpd2midy = 0;
		        Double_t loony = 0;
			Double_t looffy = 0;
                        Double_t lomidy = 0;
                       	Double_t hpd1onx = 0;
                        Double_t hpd1offx = 0;
                        Double_t hpd1midx = 0;
                        Double_t hpd2onx = 0;
                        Double_t hpd2offx = 0;
                        Double_t hpd2midx = 0;
		        Double_t loonx = 0;
			Double_t looffx = 0;
                        Double_t lomidx = 0;

			//Find out how many HPDs/L0 boards to process
			UInt_t nHPD = 0;
			UInt_t nL0 = 0;
			UInt_t nObjs = 0;
			Bool_t firstHPD = true;
			TCanvas *locanv;
			locanv = new TCanvas("locanv","locanv");
			locanv->SetLeftMargin(0.15);
			TH1 *lo = 0;
		        TH1 *hpd1 = 0;
                        TH1 *hpd2 = 0;
		        TArrow *lomidarrow = 0;
                        TArrow *loonarrow = 0;
                        TArrow *looffarrow = 0;
                        TArrow *hpd1midarrow = 0;
                        TArrow *hpd1onarrow = 0;
                        TArrow *hpd1offarrow = 0;
			TArrow *hpd2midarrow = 0;
                        TArrow *hpd2onarrow = 0;
                        TArrow *hpd2offarrow = 0;
			while ( (key = (TKey*)nextkey())) {
				TObject *obj = key->ReadObj();
				//nObjs==3 means we have the L0, HPD1 and HPD2. 
				//write out combined canvas of all 3 overlaid
/*				if(nObjs == 3){
					lo->Draw("E1");
					lo->GetXaxis()->SetTitle("delay (step)");
					lo->GetYaxis()->SetTitle("Hits");
					lo->GetYaxis()->SetTitleOffset(1.5);
					hpd1->Draw("E1 SAME");
					hpd2->Draw("E1 SAME");
					locanv->Draw();
					locanv->BuildLegend(0.8,0.8,0.9,0.9);
					locanv->Update();
					loonarrow->Draw();
					looffarrow->Draw();
					lomidarrow->Draw();
					hpd1onarrow->Draw();
					hpd1offarrow->Draw();
					hpd1midarrow->Draw();
					hpd2onarrow->Draw();
					hpd2offarrow->Draw();
					hpd2midarrow->Draw();
					locanv->Update();
					locanv->Write();
					if(fabs(hpd1midx - hpd2midx) > 5.0){
						cout << "HPD midpoints differ by > 5ns! CHECK " << lo->GetName() << endl;
					}
					locanv->Print(DBFileName + "_" +lo->GetName()+".pdf");
					locanv->Print(DBFileName + "_" +lo->GetName()+".png");
					nObjs = 0;
					firstHPD=true;
					locanv = new TCanvas("locanv","locanv");
					locanv->SetLeftMargin(0.15);
				}
				*/


				if ( obj->IsA()->InheritsFrom( "TH1" ) ) {
					TString name = obj->GetName();
					nObjs++;
					TH1 *h1 = (TH1*)obj;
					if(name.BeginsWith("HPD")){
						nHPD++;
						if(firstHPD){
							hpd1 = (TH1*)h1->Clone();
							hpd1->SetMarkerColor(kBlue);
							hpd1->SetLineColor(kBlue);
							hpd1->SetStats(false);
							max = hpd1->GetMaximum();
							min = hpd1->GetMinimum(0.0);
							//max = hpd1->GetBinContent(hpd1->GetMaximumBin());
							//min = hpd1->GetBinContent(hpd1->GetMinimumBin());
							onval = ( (max - min) * onfrac) + min;
							//cout << name << "		" << min << "	" << max << "	" << onval << endl;
							hpd1ony = hpd1->GetBinContent(hpd1->FindFirstBinAbove(onval,1));
							hpd1onx = hpd1->GetBinCenter(hpd1->FindFirstBinAbove(onval,1));
							hpd1offy = hpd1->GetBinContent(hpd1->FindLastBinAbove(onval,1));
							hpd1offx = hpd1->GetBinCenter(hpd1->FindLastBinAbove(onval,1));
							hpd1midx = (((double)hpd1offx - (double)hpd1onx) / 2.0) + (double)hpd1onx;
							hpd1midy = hpd1->GetBinContent(hpd1->FindBin(hpd1midx));
							hpd1onarrow = new TArrow(hpd1onx, hpd1ony, hpd1onx, 0.0, 0.01, ">");
							hpd1onarrow->SetLineWidth(2);
							hpd1onarrow->SetLineColor(kBlue);
							hpd1offarrow = new TArrow(hpd1offx, hpd1offy, hpd1offx, 0.0, 0.01, ">");
							hpd1offarrow->SetLineWidth(2);
							hpd1offarrow->SetLineColor(kBlue);
							hpd1midarrow = new TArrow(hpd1midx, hpd1midy, hpd1midx, 0.0, 0.01, ">");
							hpd1midarrow->SetLineWidth(2);
							hpd1midarrow->SetLineStyle(kDashed);
							hpd1midarrow->SetLineColor(kBlue);
							firstHPD = false;
						}else{
							hpd2 = (TH1*)h1->Clone();
							hpd2->SetMarkerColor(kRed);
							hpd2->SetLineColor(kRed);
							hpd2->SetStats(false);
							max = hpd2->GetMaximum();
							min = hpd2->GetMinimum(0.0);
							//max = hpd2->GetBinContent(hpd2->GetMaximumBin());
							//min = hpd2->GetBinContent(hpd2->GetMinimumBin());
							onval = ( (max - min) * onfrac) + min;
							//cout << name << "		" << min << "	" << max << "	" << onval << endl;
							hpd2ony = hpd2->GetBinContent(hpd2->FindFirstBinAbove(onval,1));
							hpd2onx = hpd2->GetBinCenter(hpd2->FindFirstBinAbove(onval,1));
							hpd2offy = hpd2->GetBinContent(hpd2->FindLastBinAbove(onval,1));
							hpd2offx = hpd2->GetBinCenter(hpd2->FindLastBinAbove(onval,1));
							hpd2midx = (((double)hpd2offx - (double)hpd2onx) / 2.0) + (double)hpd2onx;
							hpd2midy = hpd2->GetBinContent(hpd2->FindBin(hpd2midx));
							hpd2onarrow = new TArrow(hpd2onx, hpd2ony, hpd2onx, 0.0, 0.01, ">");
							hpd2onarrow->SetLineWidth(2);
							hpd2onarrow->SetLineColor(kRed);
							hpd2offarrow = new TArrow(hpd2offx, hpd2offy, hpd2offx, 0.0, 0.01, ">");
							hpd2offarrow->SetLineWidth(2);
							hpd2offarrow->SetLineColor(kRed);
							hpd2midarrow = new TArrow(hpd2midx, hpd2midy, hpd2midx, 0.0, 0.01, ">");
							hpd2midarrow->SetLineWidth(2);
							hpd2midarrow->SetLineStyle(kDashed);
							hpd2midarrow->SetLineColor(kRed);
						}
					}
					if(name.BeginsWith("L0")){
						nL0++;
						locanv->SetName(name);
						locanv->SetTitle(name);
						lo = (TH1*)h1->Clone();
						lo->SetStats(false);
						max = lo->GetMaximum();
						min = lo->GetMinimum(0.0);
						//max = lo->GetBinContent(lo->GetMaximumBin());
						//min = lo->GetBinContent(lo->GetMinimumBin());
						onval = ( (max - min) * onfrac) + min;
						//cout << name << "		" << min << "	" << max << "	" << onval << endl;
						loony = lo->GetBinContent(lo->FindFirstBinAbove(onval,1));
						loonx = lo->GetBinCenter(lo->FindFirstBinAbove(onval,1));
						looffy = lo->GetBinContent(lo->FindLastBinAbove(onval,1));
						looffx = lo->GetBinCenter(lo->FindLastBinAbove(onval,1));
						lomidx = (((double)looffx - (double)loonx) / 2.0) + (double)loonx;
						lomidy = lo->GetBinContent(lo->FindBin(lomidx));
						loonarrow = new TArrow(loonx, loony, loonx, 0.0, 0.01, ">");
						loonarrow->SetLineWidth(2);
						looffarrow = new TArrow(looffx, looffy, looffx, 0.0, 0.01, ">");
						looffarrow->SetLineWidth(2);
						lomidarrow = new TArrow(lomidx, lomidy, lomidx, 0.0, 0.01, ">");
						lomidarrow->SetLineWidth(2);
						lomidarrow->SetLineStyle(kDashed);

					}

				if(nObjs == 3){
					lo->Draw("E1");
					lo->GetXaxis()->SetTitle("delay (step)");
					lo->GetYaxis()->SetTitle("Hits");
					lo->GetYaxis()->SetTitleOffset(1.5);
					hpd1->Draw("E1 SAME");
					hpd2->Draw("E1 SAME");
					locanv->Draw();
					locanv->BuildLegend(0.8,0.8,0.9,0.9);
					locanv->Update();
					loonarrow->Draw();
					looffarrow->Draw();
					lomidarrow->Draw();
					hpd1onarrow->Draw();
					hpd1offarrow->Draw();
					hpd1midarrow->Draw();
					hpd2onarrow->Draw();
					hpd2offarrow->Draw();
					hpd2midarrow->Draw();
					locanv->Update();
					locanv->Write();
					if(fabs(hpd1midx - hpd2midx) > 5.0){
						cout << "HPD midpoints differ by > 5ns! CHECK " << lo->GetName() << endl;
					}
					locanv->Print(DBFileName + "_" +lo->GetName()+".pdf");
					locanv->Print(DBFileName + "_" +lo->GetName()+".png");
					nObjs = 0;
					firstHPD=true;
					locanv = new TCanvas("locanv","locanv");
					locanv->SetLeftMargin(0.15);
				}
				}
			}


			TH1F *l0barsmid = new TH1F("L0 alignment plot (Midpoints) "+RichName,"L0 alignment plot (Midpoints) "+RichName,nL0,0,nL0);
			TH1F *l0barson = new TH1F("L0 alignment plot (Turn-on points) "+RichName,"L0 alignment plot (Turn-on points) "+RichName,nL0,0,nL0);
			TH1F *l0barsoff = new TH1F("L0 alignment plot (Turn-off points) "+RichName,"L0 alignment plot (Turn-off points) "+RichName,nL0,0,nL0);
			TH1F *HPDbarsmid = new TH1F("HPD alignment plot (Midpoints) "+RichName,"HPD alignment plot (Midpoints) "+RichName,nHPD,0,nHPD);
			TH1F *HPDbarson = new TH1F("HPD alignment plot (Turn-on points) "+RichName,"HPD alignment plot (Turn-on points) "+RichName,nHPD,0,nHPD);
			TH1F *HPDbarsoff = new TH1F("HPD alignment plot (Turn-off points) "+RichName,"HPD alignment plot (Turn-off points) "+RichName,nHPD,0,nHPD);


			UInt_t HPD = 0;
			UInt_t L0 = 0;
			cout << "Histo	Turn on	Turn off	delta	midpoint" << endl;
			TIter nextkey2( current_sourcedir->GetListOfKeys() );
			TKey *key2;
			while ( (key2 = (TKey*)nextkey2())) {

				TObject *obj = key2->ReadObj();
				if ( obj->IsA()->InheritsFrom( "TH1" ) ) {
					// descendant of TH1 -> find turn-on point
					TString *name = new TString(obj->GetName());
					TH1 *h1 = (TH1*)obj;
					if(h1->GetEntries() !=0){
						max = h1->GetMaximum();
						min = h1->GetMinimum(0.0);
						delta = max - min;
						onval = ( delta * onfrac) + min;
						turnon = h1->GetBinCenter(h1->FindFirstBinAbove(onval,1));
						turnoff = h1->GetBinCenter(h1->FindLastBinAbove(onval,1));
						midpoint = (((double)turnoff - (double)turnon) / 2.0) + (double)turnon; 
						if(!madeAxes){
							Int_t bins = h1->GetXaxis()->GetNbins();
							//cout << bins << endl;
							Double_t lower = h1->GetXaxis()->GetXmin();
							//cout << lower << endl;
							Double_t upper = h1->GetXaxis()->GetXmax();
							//cout << upper << endl;
							if(argc!=7){
							minbin=lower;
							maxbin=upper;
							}

							l0barsmid->GetYaxis()->SetRangeUser(minbin,maxbin);
							l0barson->GetYaxis()->SetRangeUser(minbin,maxbin);
							l0barsoff->GetYaxis()->SetRangeUser(minbin,maxbin);
							HPDbarsmid->GetYaxis()->SetRangeUser(minbin,maxbin);
							HPDbarson->GetYaxis()->SetRangeUser(minbin,maxbin);
							HPDbarsoff->GetYaxis()->SetRangeUser(minbin,maxbin);

							
							globalHits = new TH1F("Global hits per step "+RichName,"Global hits per step "+RichName,bins,lower,upper);
							hpdon = new TH1I("HPD turn-on profile "+RichName,"HPD turn-on profile "+RichName,bins,lower,upper);
							hpdmid = new TH1F("HPD midpoint profile "+RichName,"HPD midpoint profile "+RichName,bins,lower,upper);
							hpdoff = new TH1I("HPD turn-off profile "+RichName,"HPD turn-off profile "+RichName,bins,lower,upper);
							l0on = new TH1I("L0 turn-on profile "+RichName,"L0 turn-on profile "+RichName,bins,lower,upper);
							l0off = new TH1I("L0 turn-off profile "+RichName,"L0 turn-off profile "+RichName,bins,lower,upper);
							l0mid = new TH1F("L0 midpoint profile "+RichName,"L0 midpoint profile "+RichName,bins,lower,upper);

							hpdon->GetXaxis()->SetTitle("Turn on (step)");
							hpdon->GetYaxis()->SetTitle("HPDs");
							hpdmid->GetXaxis()->SetTitle("Midpoint (step)");
							hpdmid->GetYaxis()->SetTitle("HPDs");
							hpdoff->GetXaxis()->SetTitle("Turn off (step)");
							hpdoff->GetYaxis()->SetTitle("HPDs");

							l0on->GetXaxis()->SetTitle("Turn on (step)");
							l0on->GetYaxis()->SetTitle("L0 boards");
							l0mid->GetXaxis()->SetTitle("Midpoint (step)");
							l0mid->GetYaxis()->SetTitle("L0 boards");
							l0off->GetXaxis()->SetTitle("Turn off (step)");
							l0off->GetYaxis()->SetTitle("L0 boards");

							HPDbarson->GetYaxis()->SetTitle("delay (step)");
							HPDbarsoff->GetYaxis()->SetTitle("delay (step)");
							HPDbarsmid->GetYaxis()->SetTitle("delay (step)");
							l0barson->GetYaxis()->SetTitle("delay (step)");
							l0barsoff->GetYaxis()->SetTitle("delay (step)");
							l0barsmid->GetYaxis()->SetTitle("delay (step)");

							globalHits->GetYaxis()->SetTitle("Hits");
							globalHits->GetXaxis()->SetTitle("delay (step)");
							globalHits->Sumw2();

							madeAxes=true;
						}

						if(name->BeginsWith("HPD_")){
							cout << obj->GetName() << "	" << turnon << "	" << turnoff << "	" << turnoff - turnon << "	" << midpoint <<endl;
							doccupancy = sqrt(doccupancy*doccupancy*occupancy*occupancy + max*max*max)/(occupancy + max);
							occupancy += max;
							globalHits->Add(h1,1);

							hpdon->Fill(turnon);
							hpdoff->Fill(turnoff);
							hpdmid->Fill(midpoint);
							//HPDbarson->GetXaxis()->SetBinLabel(HPD+1,name->Data());
							//HPDbarsoff->GetXaxis()->SetBinLabel(HPD+1,name->Data());
							//HPDbarsmid->GetXaxis()->SetBinLabel(HPD+1,name->Data());
							HPDbarson->Fill(name->Data(),turnon);
							HPDbarsoff->Fill(name->Data(),turnoff);
							HPDbarsmid->Fill(name->Data(),midpoint);

							HPDavemidpoint += midpoint;
							HPD++;
							HPDIDs.push_back(name->Remove(0,3));
							HPDmidpoints.push_back(midpoint);


						}
						if(name->BeginsWith("L0_")){
							cout << obj->GetName() << "		" << turnon << "	" << turnoff << "	" << turnoff - turnon << "	" << midpoint <<endl;
							l0on->Fill(turnon);
							l0off->Fill(turnoff);
							l0mid->Fill(midpoint);
							//l0barson->GetXaxis()->SetBinLabel(L0+1,name->Data());
							//l0barsoff->GetXaxis()->SetBinLabel(L0+1,name->Data());
							//l0barsmid->GetXaxis()->SetBinLabel(L0+1,name->Data());
							l0barson->Fill(name->Data(),turnon);
							l0barsoff->Fill(name->Data(),turnoff);
							l0barsmid->Fill(name->Data(),midpoint);
							L0avemidpoint += midpoint;
							L0++;
							L0IDs.push_back(name->Remove(0,3));
							L0midpoints.push_back(midpoint);

						}
					}else{
						cout << *name << " is empty! " << endl;
					}

				}else{
					cout << "This is not a TH1!" << endl;
				}

			}


			HPDbarson->LabelsDeflate("X");
			HPDbarsoff->LabelsDeflate("X");
			HPDbarsmid->LabelsDeflate("X");
			l0barson->LabelsDeflate("X");
			l0barsoff->LabelsDeflate("X");
			l0barsmid->LabelsDeflate("X");



			//			HPDIDs.resize(HPD);
			//			HPDmidpoints.resize(HPD);
			//			L0IDs.resize(L0);
			//			L0midpoints.resize(L0);

			Double_t aveOccupancy = occupancy/(Double_t)HPD;
			Double_t daveOccupancy = doccupancy/(Double_t)HPD;	

			L0avemidpoint = L0avemidpoint/(Double_t)L0;
			HPDavemidpoint = HPDavemidpoint/(Double_t)HPD;

			cout << "Number of HPDs found/counted = " << nHPD << "/" << HPD << endl;
			cout << "Number of L0 found/counted = " << nL0 << "/" << L0 << endl;
			cout << "ave. midpoint (HPDs) = " << HPDavemidpoint << endl;
			cout << "ave. midpoint (L0) = " << L0avemidpoint << endl;
			cout << "ave. max. occupancy = " << aveOccupancy << " +/- " << daveOccupancy << endl;

			outputdir->cd();
			l0on->Write();
			l0off->Write();
			l0mid->Write();
			hpdon->Write();
			hpdoff->Write();
			hpdmid->Write();
			globalHits->Write();

			TCanvas *l0bars = new TCanvas("L0 alignment plot "+RichName,"L0 alignment plot "+RichName,2048,768);
			TLegend *l0legend = new TLegend(0.75,0.75,0.85,0.85);
			l0barson->SetStats(false);
			l0barsoff->SetStats(false);
			l0barsmid->SetStats(false);
			l0barsmid->GetXaxis()->SetLabelSize(0.03);
			l0barsmid->SetMarkerStyle(kStar);
			l0barsmid->SetMarkerColor(kGreen);
			l0barson->GetXaxis()->SetLabelSize(0.03);
			l0barson->SetMarkerStyle(kFullSquare);
			l0barsoff->GetXaxis()->SetLabelSize(0.03);
			l0barsoff->SetMarkerStyle(kOpenSquare);
			l0barson->SetMarkerColor(kBlue);
			l0barsoff->SetMarkerColor(kRed);
			l0barsmid->SetMarkerSize(0.5);
			l0barson->SetMarkerSize(0.5);
			l0barsoff->SetMarkerSize(0.5);
			l0legend->AddEntry(l0barson,"Turn on","p");
			l0legend->AddEntry(l0barsmid,"Midpoint","p");
			l0barson->SetTitle("L0 Time Alignment Plot, "+RichName);
			l0barson->Draw("hist,p");
			l0barsoff->Draw("hist,p,same");
			l0barsmid->Draw("hist,p,same");
			l0legend->AddEntry(l0barsoff,"Turn off","p");
			l0legend->SetShadowColor(0);
			l0legend->SetLineColor(1);
			l0legend->SetTextSize(0.03);
			l0legend->SetFillColor(0);
			l0legend->Draw();
			l0bars->Write();

			TCanvas *HPDbars = new TCanvas("HPD alignment plot "+RichName,"HPD alignment plot "+RichName,2048,768);
			TLegend *HPDlegend = new TLegend(0.75,0.75,0.85,0.85);

			HPDbarson->SetStats(false);
			HPDbarsoff->SetStats(false);
			HPDbarsmid->SetStats(false);
			HPDbarsmid->GetXaxis()->SetLabelSize(0.03);
			HPDbarson->GetXaxis()->SetLabelSize(0.03);
			HPDbarsoff->GetXaxis()->SetLabelSize(0.03);
			HPDbarsmid->SetMarkerStyle(kStar);
			HPDbarson->SetMarkerStyle(kFullSquare);
			HPDbarsoff->SetMarkerStyle(kOpenSquare);
			HPDbarsmid->SetMarkerSize(0.5);
			HPDbarson->SetMarkerSize(0.5);
			HPDbarsoff->SetMarkerSize(0.5);
			HPDbarsmid->SetMarkerColor(kGreen);
			HPDbarson->SetMarkerColor(kBlue);
			HPDbarsoff->SetMarkerColor(kRed);
			HPDbarson->SetTitle("HPD Time Alignment Plot, "+RichName);
			HPDbarson->Draw("hist,p");
			HPDlegend->AddEntry(HPDbarson,"Turn on","p");
			HPDbarsoff->Draw("hist,p,same");
			HPDlegend->AddEntry(HPDbarsoff,"Turn off","p");
			HPDbarsmid->Draw("hist,p,same");
			HPDlegend->AddEntry(HPDbarsmid,"Midpoint","p");
			HPDlegend->SetShadowColor(0);
			HPDlegend->SetLineColor(1);
			HPDlegend->SetTextSize(0.03);
			HPDlegend->SetFillColor(0);
			HPDlegend->Draw();
			HPDbars->Write();

			TCanvas *HPDmids = new TCanvas("HPD midpoints "+RichName,"HPD midpoints "+RichName,1024,768);
			hpdmid->Draw();
			HPDmids->Write();

			TCanvas *L0mids = new TCanvas("L0 midpoints "+RichName,"L0 midpoints "+RichName,1024,768);
			l0mid->Draw();
			L0mids->Write();

			TCanvas *Globalhits = new TCanvas("Global Hits "+RichName,"Global Hits "+RichName,1024,768);
			globalHits->Draw();
			Globalhits->Write();



			HPDmids->Print(DBFileName + "_HPD_midpoints.png");
			L0mids->Print(DBFileName + "_L0_midpoints.png");
			HPDbars->Print(DBFileName + "_HPD_alignment.png");
			l0bars->Print(DBFileName + "_L0_alignment.png");
			HPDmids->Print(DBFileName + "_HPD_midpoints.pdf");
			L0mids->Print(DBFileName + "_L0_midpoints.pdf");
			HPDbars->Print(DBFileName + "_HPD_alignment.pdf");
			l0bars->Print(DBFileName + "_L0_alignment.pdf");
			Globalhits->Print(DBFileName + "_GlobalHits.png");
			Globalhits->Print(DBFileName + "_GlobalHits.pdf");

			DBFileName += ".txt";
			cout << "# Writing DBFile " << DBFileName << endl; 
			std::ofstream DBfile (DBFileName.Data());
			if (DBfile.is_open()){
				Int_t L0midfloor;
				for(UInt_t i = 0; i<L0; i++){
					L0midfloor = (Int_t)TMath::Floor(L0midpoints[i]);
					DBfile << L0IDs[i] << ".TTCRx.Settings." << scanType << ":" << L0midfloor << "\n";
				}
				DBfile.close();
			}else{
				cout << "Error writing to " << outputname << endl;
			}
		}
	}
	output->Close();
}
