#!/bin/env python

from ROOT import gROOT, TH1F, TFile, TCanvas, TLatex
import os
import argparse

parser = argparse.ArgumentParser(description='Make some official plots')
parser.add_argument('--stylefile', help='location of a style file to process instead of lhcbStyle.C shipped with this')
parser.add_argument('--rootfile', required=True,help='the output of the timing scan script')
parser.add_argument('--outfile', help='the name of an output file to save to')
parser.add_argument('--outprefix', help='the prefix for the output files if individual files required')
parser.add_argument('--outformat', help='the extension for the output files if individual files required')


args = parser.parse_args()

try:
  infile = TFile(args.rootfile)
except:
  print 'Could not open', args.rootfile
  quit()

if infile.IsZombie():
  print 'Could not open', args.rootfile
  quit()

outfilename = 'scan_official.pdf'
if args.outfile:
  if args.outfile == 'None':
    outfilename = None
  else:
    outfilename = args.outfile

outprefix = None
outformat = None
if args.outprefix:
  if args.outformat:
    outprefix = args.outprefix
    outprefix = args.outprefix
  else:
    print 'You have not specified a format for individual files'

gROOT.ProcessLine('.X ' + os.path.dirname(os.path.realpath(__file__)) + '/lhcbStyle.C')
hrich1 = infile.Get("RICH1/L0 midpoint profile RICH1")
hrich2 = infile.Get("RICH2/L0 midpoint profile RICH2")

hrich1.GetXaxis().SetTitle('#deltat_{trigger-photons} (ns)')
hrich2.GetXaxis().SetTitle('#deltat_{trigger-photons} (ns)')

canvas = TCanvas()
if outfilename:
  canvas.SaveAs(outfilename+'[')

hrich1.Draw()
label=TLatex(8, 0.7*hrich1.GetMaximum(), '#splitline{'+'Mean = {:.2f} ns'.format(hrich1.GetMean()) +'}{'+'RMS  = {:.2f}'.format(hrich1.GetRMS())+' ns}')
label.Draw()
if outfilename:
  canvas.SaveAs(outfilename)
if outprefix and outformat:
  canvas.SaveAs(outprefix + '_LO_midpoint_RICH1.' + outformat)

hrich2.Draw()
label=TLatex(8, 0.7*hrich2.GetMaximum(), '#splitline{'+'Mean = {:.2f} ns'.format(hrich2.GetMean()) +'}{'+'RMS  = {:.2f}'.format(hrich2.GetRMS())+' ns}')
label.Draw()
if outfilename:
  canvas.SaveAs(outfilename)
if outprefix and outformat:
  canvas.SaveAs(outprefix + '_LO_midpoint_RICH2.' + outformat)


if outfilename:
  canvas.SaveAs(outfilename+']')


