// $Id: $
#ifndef RICHTIMINGSCANANALYSIS_H
#define RICHTIMINGSCANANALYSIS_H 1

#include <sstream>

// from Gaudi
#include "RichKernel/RichHistoAlgBase.h"
#include "GaudiAlg/GaudiAlgorithm.h"

#include "RichInterfaces/IRichSmartIDTool.h"
#include "RichInterfaces/IRichRawBufferToSmartIDsTool.h"
#include "RichDet/DeRichSystem.h"
#include "RichInterfaces/IRichPixelClusteringTool.h"

#include "AIDA/IHistogram1D.h"
#include "AIDA/IHistogram2D.h"

#include "RichRecBase/RichRecHistoAlgBase.h"

/** @class RichTimingScanAnalysis RichTimingScanAnalysis.h
 *
 *
 *  @author Nicholas Styles
 *  @date   2007-12-02
 *  @author Conor Fitzpatrick
 *  @date   2009-10-02
 */
class RichTimingScanAnalysis : public Rich::HistoAlgBase {
public:
  /// Standard constructor
  RichTimingScanAnalysis( const std::string& name, ISvcLocator* pSvcLocator );

  virtual ~RichTimingScanAnalysis( ); ///< Destructor

  StatusCode initialize() override;    ///< Algorithm initialization
  StatusCode execute   () override;    ///< Algorithm execution

private:
   virtual StatusCode book(); ///< Prepare and book histos
  bool  checkODINBxID(); //Cut on BxIds

private:

  typedef std::map<int,int> L0IDMap;
  L0IDMap m_id;

//  int m_profile[288][26];
//  int m_onoff[288][2];

  const Rich::DAQ::IRawBufferToSmartIDsTool * m_SmartIDDecoder;
  const Rich::ISmartIDTool * m_idTool;
  const Rich::DAQ::IPixelClusteringTool * m_clusterTool;
  const DeRichSystem * m_RichSys;
  int m_EmptyEvents;
  unsigned int m_nEventsPerStep;
  unsigned int m_nSteps;
  int m_stepWidth;
  int m_stepStart;
  unsigned int m_minClusSize;
  unsigned int m_maxClusSize;
  unsigned int m_whichRich;
  bool m_filterBX;
  std::vector<long> m_bxIDs;

  std::map< Rich::DAQ::HPDHardwareID, IHistogram1D* > m_HPDtimingHists;
  std::map< Rich::DAQ::ShortType, IHistogram1D* > m_L0timingHists;

  int m_totHits;
  int m_evt;
  int m_step;
  int totCorrection;
};
#endif // RICHTIMINGSCANANALYSIS_H
